/**--------------------------------------------------------------------------
 **
 **		NEC PC-8001 Emulator "N80" for Win32/DirectX
 **		--------------------------------------------
 **		Copyright (c) 1993-2020 H.Kanemoto
 **
 **		Main procedure
 **
 **		Rev		Date		Descriptions
 **		-----	----------	-----------------------------------------------
 **		1.00	1996/06/27	porting DOS version on WinG
 **		1.20	1997/08/10	porting WinG version on Win32/DirectX
 **		1.30	1998/03/12	
 **		1.43	2007/09/01	XPΉ
 **		1.44	2009/12/15	Vista/7Ή
 **		1.51	2010/01/27	DirectSound OnlyɕύX
 **		1.76	2020/09/29	PCG 3ch Sound (PCGfΉ)
 **------------------------------------------------------------------------*/
#include "n80/n80_form.h"
#include "n80/n80_directx.h"
#include "z80/z80_thread.h"
#include "res/resource.h"
#define export
/**--------------------------------------------------------------------------
 **		W[ϐ
 **------------------------------------------------------------------------*/
export N80				n80win32;	// N80 win32
export N80DEVICE		n80device;	// PC-8001 Devices
static LPCTSTR			cCLSNAME = TEXT("IICCJ");
export LPCTSTR			cAPPNAME = TEXT("N80 - PC-8001 Emulator");
export OSVERSIONINFO	osVer;
export DWORD_PTR		affinity = 1;
export n80_assay		*assay = NULL;
export n80_form			*form = NULL;
export n80_directx		*dx = NULL;
export z80_thread		*th = NULL;
static HANDLE			sndmutex = NULL;
static int				n80main(HINSTANCE);
/**--------------------------------------------------------------------------
 **		CJn
 **------------------------------------------------------------------------*/
#if _MSC_VER <= 1310	// ꂪʂ̂VS2003܂ŁHH
int APIENTRY
WinMain (HINSTANCE hIns, HINSTANCE hPrev, LPTSTR cmdln, int cmdsw)
#else
int WINAPI WinMain (
    __in HINSTANCE hIns,
    __in_opt HINSTANCE hPrev,
    __in_opt LPSTR cmdln,
    __in int cmdsw
    )
#endif
{
	// dNh~
	HANDLE mutex = ::CreateMutex(NULL, TRUE, "N80MultiBootChecker");
	if (::GetLastError() == ERROR_ALREADY_EXISTS) {
		assay->outputLog(TEXT("N80 dNm"));
		::ReleaseMutex(mutex);
		::CloseHandle(mutex);
		return 1;
	}
	// OS擾
	::ZeroMemory(&osVer, sizeof(OSVERSIONINFO));
	osVer.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	::GetVersionEx(&osVer);

	assay = new n80_assay();	// N80 assay
	form = new n80_form(hIns);	// N80 Form
	dx = new n80_directx();		// N80 DirectX
	th = new z80_thread();		// Z80 thread

	assay->outputLog(TEXT("------------------ N80 start ----------------"));
	int result = n80main(hIns);	// N80 main procedure
	assay->outputLog(TEXT("------------------ N80 end ------------------"));

	delete dx;					// release N80 DirectX
	delete th;					// release Z80 thread
	delete form;				// release N80 Form
	delete assay;				// release N80 assay

	/*  Mutex̊J */
	::ReleaseMutex(mutex);
	::CloseHandle(mutex);
	return result;
}

static int n80main(HINSTANCE hIns)
{
	int result = 1;

	// Process Affinity擾
	DWORD_PTR paffmask, saffmask;
	if (::GetProcessAffinityMask(::GetCurrentProcess(), &paffmask, &saffmask)) {
		affinity = paffmask;
	}
	::SetThreadAffinityMask(::GetCurrentThread(), 1);

	// x1msɐݒ
	::timeBeginPeriod (1);

	// ICON\[X擾
	HICON hicon = ::LoadIcon (hIns, MAKEINTRESOURCE(IDI_ICON1));

	// COM
	if (FAILED(::CoInitialize(NULL))) {
		assay->outputLog(TEXT("ERROR CoInitializes"));
		::MessageBox(NULL, TEXT("CoInitializes"), cAPPNAME, MB_ICONSTOP | MB_OK);
	// EBhENXo^
	} else if (!n80_form::registerClass(hIns, hicon, NULL, NULL, cCLSNAME)) {
		assay->outputLog(TEXT("ERROR RegisterClassExs"));
		::MessageBox(NULL, TEXT("RegisterClassExs"), cAPPNAME, MB_ICONSTOP | MB_OK);
	// EBhE
	} else if (!form->create(cCLSNAME, cAPPNAME, 0, 0, 640, 480)) {
		assay->outputLog(TEXT("ERROR CreateWindowExs"));
		::MessageBox(NULL, TEXT("CreateWindowExs"), cAPPNAME, MB_ICONSTOP | MB_OK);
	} else if ((sndmutex = ::CreateMutex(NULL, FALSE, "SNDMUTEX")) == NULL) {
		assay->outputLog(TEXT("ERROR CreateMutexs"));
		::MessageBox(NULL, TEXT("CreateMutexs"), cAPPNAME, MB_ICONSTOP | MB_OK);
	// bZ[W[vJn
	} else {
		assay->outputLog(TEXT("ENTER message Loop"));
		if (form->init())
			result = form->messageLoop();
		assay->outputLog(TEXT("LEAVE message Loop"));
	}
	
	// x
	::timeEndPeriod (1);
	
	// Mutex
	::CloseHandle(sndmutex);

	// COM
	::CoUninitialize ();

	return result;
}
