/**--------------------------------------------------------------------------
 **
 **		NEC PC-8001 Emulator "N80" for Win32/DirectX
 **		--------------------------------------------
 **		Copyright (c) 1993-2020 H.Kanemoto
 **
 **		Direct Sound
 **
 **		Rev		Date		Descriptions
 **		-----	----------	-----------------------------------------------
 **		1.00	1996/06/27	porting DOS version on WinG
 **		1.20	1997/08/10	porting WinG version on Win32/DirectX
 **		1.30	1998/03/12	
 **		1.43	2007/09/01	XPΉ
 **		1.44	2009/12/15	Vista/7Ή
 **		1.51	2010/01/27	DirectSound OnlyɕύX
 **		1.61	2010/10/31	obt@]Offset𑵂悤ύX
 **		1.76	2020/09/29	PCG 3ch Sound (PCGfΉ)
 **------------------------------------------------------------------------*/
#include <crtdbg.h>
#include "n80_directx.h"
#include "n80_form.h"
#include "../z80/z80_thread.h"
#include "../lib/icj_memory.h"
#include <stdio.h>
#include <math.h>
/**--------------------------------------------------------------------------
 **		O[oϐ
 **------------------------------------------------------------------------*/
extern DWORD_PTR	affinity;
extern n80_assay	*assay;
extern z80_thread	*th;
extern N80			n80win32;
/**--------------------------------------------------------------------------
 **		DirectSound 
 **------------------------------------------------------------------------*/
BOOL
n80_directx::init_ds() {
	/**----------------------------------------------------------------------
	 **		DirectSoundIuWFNg
	 **--------------------------------------------------------------------*/
	if (FAILED(::DirectSoundCreate(NULL, &m_ds, NULL))) {
		assay->outputLog(TEXT("ERROR DS:SetCooperativeLevels"));
		m_ds = NULL;
		return FALSE;
	}
	// xݒ
	if (FAILED(m_ds->SetCooperativeLevel(m_hwnd, DSSCL_PRIORITY))) {
		assay->outputLog(TEXT("ERROR DS:SetCooperativeLevels"));
		goto exit_by_error;
	}
	/**----------------------------------------------------------------------
	 **		DirectSound Primary SoundBuffer
	 **--------------------------------------------------------------------*/
	::ZeroMemory(&m_dsbdesk, sizeof(m_dsbdesk));
	m_dsbdesk.dwSize  = sizeof(m_dsbdesk);
	m_dsbdesk.dwFlags = DSBCAPS_PRIMARYBUFFER	// create primary buffer
					  |	DSBCAPS_LOCHARDWARE		// Hardware Mixing
					  |	DSBCAPS_CTRLVOLUME;		// Enable Volume change
	//				  |	DSBCAPS_CTRLPAN;		// Enable Pan
	if (FAILED(m_ds->CreateSoundBuffer (&m_dsbdesk, &m_dssb1, NULL))) {
		assay->outputLog(TEXT("ERROR DS:CreateSoundBuffer(Primary)s"));
		goto exit_by_error;
	}
	// Primary Buffer Size擾
	DSBCAPS caps;
	::ZeroMemory(&caps, sizeof(caps));
	caps.dwSize = sizeof(caps);
	if (FAILED(m_dssb1->GetCaps(&caps))) {
		assay->outputLog(TEXT("ERROR DS:GetCaps(Primary)s"));
		goto exit_by_error;
	}
	DWORD IZEPRIMBUF = caps.dwBufferBytes;
	assay->outputLog(TEXT("DirectSound: Primary Buffer size = %d"), IZEPRIMBUF);
	// set Primary SoundBuffer format
	WAVEFORMATEX wfx;
	::ZeroMemory(&wfx, sizeof(wfx));
	wfx.cbSize = sizeof(wfx);
	wfx.wFormatTag = WAVE_FORMAT_PCM;	// PCM
	wfx.nChannels = CHANNELS;			// 1ch MONO
	wfx.nSamplesPerSec = SMPLRATE;		// sampling rate = 96kHz
	wfx.wBitsPerSample = BITPERSAMPLE;	// ʎqrbg(16bit)
	wfx.nBlockAlign = BLOCKALIGN;		// bytes per one sampling (2bytes[16bit]~1ch)
	wfx.nAvgBytesPerSec = BYTEPERSEC;	// length per one second (2bytes[16bit]~1ch~96000Hz)
	if (FAILED(m_dssb1->SetFormat(&wfx))) {
		assay->outputLog(TEXT("ERROR DS:SetFormat(Primary)s"));
		goto exit_by_error;
	}
	/**----------------------------------------------------------------------
	 **		DirectSound Secondary SoundBuffer
	 **--------------------------------------------------------------------*/
	::ZeroMemory(&wfx, sizeof(wfx));
	wfx.cbSize = sizeof(wfx);
	wfx.wFormatTag = WAVE_FORMAT_PCM;	// PCM
	wfx.nChannels = CHANNELS;			// 1ch MONO
	wfx.nSamplesPerSec = SMPLRATE;		// sampling rate = 96kHz
	wfx.wBitsPerSample = BITPERSAMPLE;	// ʎqrbg(16bit)
	wfx.nBlockAlign = BLOCKALIGN;		// bytes per one sampling (16bit~1ch)
	wfx.nAvgBytesPerSec = BYTEPERSEC;	// length per one second (16bit~1ch~96000Hz)

	::ZeroMemory(&m_dsbdesk, sizeof(m_dsbdesk));
	m_dsbdesk.dwSize  = sizeof(m_dsbdesk);
	m_dsbdesk.lpwfxFormat = &wfx;
	m_dsbdesk.dwFlags = DSBCAPS_GETCURRENTPOSITION2 // Đʒu̎擾L
					  |	DSBCAPS_CTRLPOSITIONNOTIFY	// ĐʒuʒmCxgL
					  |	DSBCAPS_LOCDEFER
					  |	DSBCAPS_CTRLPOSITIONNOTIFY
					  |	DSBCAPS_GLOBALFOCUS
					  | DSBCAPS_STICKYFOCUS;
	//				  |	DSBCAPS_CTRLVOLUME;
	m_dsbdesk.dwBufferBytes = DELAYBUFSIZE*4;			// obt@̃TCY(20x4msec)
	if (FAILED(m_ds->CreateSoundBuffer(&m_dsbdesk, &m_dssb2, NULL))) {		
		assay->outputLog(TEXT("ERROR DS:CreateSoundBuffer(2nd)s"));
		goto exit_by_error;
	}
	/**----------------------------------------------------------------------
	 **		DirectSound SoundBuffer Layout
	 **
	 **		|<--------------------- 40ms -------------------->|
	 **		|------------------------|                        |
	 **		|                        |
	 **		|<-------- 20ms -------->| <= PCM Make size
	 **--------------------------------------------------------------------*/
	::ZeroMemory(&caps, sizeof(caps));
	caps.dwSize = sizeof(caps);
	if (FAILED(m_dssb2->GetCaps(&caps))) {		
		assay->outputLog(TEXT("ERROR DS:GetCaps(2nd)s"));
		goto exit_by_error;
	}
	/**----------------------------------------------------------------------
	 **		SoundBuffer Clear and start to Play (Beep)
	 **--------------------------------------------------------------------*/
	LPVOID	lpSoundData1, lpSoundData2;
	DWORD	size1, size2;
	if (FAILED(m_dssb2->Lock(0, DELAYBUFSIZE,
					&lpSoundData1, &size1,
					&lpSoundData2, &size2,
					DSBLOCK_ENTIREBUFFER))) {
		assay->outputLog(TEXT("ERROR DS:Lock(2nd)s"));
		goto exit_by_error;
	}
	::ZeroMemory(lpSoundData1, DELAYBUFSIZE);
	m_dssb2->Unlock(lpSoundData1, size1, lpSoundData2, size2);
	m_dssb2->Stop();
	/**----------------------------------------------------------------------
	 **		eXgpϐ
	 **--------------------------------------------------------------------*/
#if defined(_TEST) || defined(_DEBUG)
	// tԊu
	assay->beginTakeInterval();
#endif
	calc_delay_ticks();
	return  TRUE;

exit_by_error:
	dispose_ds();
	return FALSE;
}

void
n80_directx::dispose_ds() {
	dispose_soundbuff();
	if (m_ds) {
		m_ds->Release();
		m_ds = NULL;
	}
}

void
n80_directx::dispose_soundbuff() {
	if (m_dssb1) {
		m_dssb1->Release();
		m_dssb1 = NULL;
	}
	if (m_dssb2) {

		m_dssb2->Release();
		m_dssb2 = NULL;
	}
}
/**--------------------------------------------------------------------------
 **		DirectSound Preference
 **------------------------------------------------------------------------*/
void
n80_directx::calc_delay_ticks() {
	m_tick_delay = (LONGLONG) (n80device.param.hz * 20 / 1000);	// T-states per 20ms
	assay->outputLog(TEXT("reset_cpu_clk: m_tick_delay=%dl"), m_tick_delay);
#if defined(_TEST) || defined(_DEBUG)
	// tԊu
	//assay->changeTakeInterval(n80device.param.hz);
#endif
}

LONG
n80_directx::getHardVolume() {
	LONG vol;
	m_dssb1->GetVolume(&vol);
	return vol / 100;
}

void
n80_directx::setHardVolume(LONG vol) {
	m_dssb1->SetVolume(vol * 100);
}

short
n80_directx::getSoftVolume() {
	return m_beep_amplitude / 1000;
}

void
n80_directx::setSoftVolume(short amp) {
	m_beep_amplitude = amp * 1000;
	m_pcg_amplitude  = (short) floor(((double) m_beep_amplitude) * 0.9);
}
/**--------------------------------------------------------------------------
 **		PC-8001 Beep Sound (Z80 Thread)
 **------------------------------------------------------------------------*/
void
n80_directx::clear_Ts_bank() {
	memset(&this->m_beep_bank,  0, sizeof(this->m_beep_bank));
	memset(&this->m_sound_bank, 0, sizeof(this->m_sound_bank));
	this->m_beep_phase_adjuster  = 0;
	this->m_sound_phase_adjuster = 0;
}

void
n80_directx::appendTs(LONGLONG tick) {
	this->m_beep_bank.mCurrent.mTicks += tick;
	this->m_sound_bank[0].mCurrent.mTicks += tick;
	this->m_sound_bank[1].mCurrent.mTicks += tick;
	this->m_sound_bank[2].mCurrent.mTicks += tick;
}

void
n80_directx::beep_on() {
	// OFF->ONgK[ł܂łOFFo^
	if (this->m_beep_bank.mCurrent.mState == 0) {
		this->appendPCMTick(&this->m_beep_bank, 1, 0);
	}
}

void
n80_directx::beep_off() {
	// ON->OFFgK[ł܂łONo^
	if (this->m_beep_bank.mCurrent.mState == 1) {
		this->appendPCMTick(&this->m_beep_bank, 0, 0);
	}
}
/**--------------------------------------------------------------------------
 **		PC-8001 PCG Sound (Z80 Thread)
 **------------------------------------------------------------------------*/
void
n80_directx::sound_on(ULONG freq, int ch) {
	DWORD state = freq > 0 ? (DWORD)floor(((double)SMPLRATE) / ((double)freq) + 0.5) : 0;
	if (this->m_sound_bank[ch].mCurrent.mState != state) {
		for (int i = 0; i < 3; ++i) {
			ULONG f = i == ch ? freq : this->m_sound_bank[i].mCurrent.mFreq;
			DWORD s = i == ch ? state : this->m_sound_bank[i].mCurrent.mState;
			this->appendPCMTick(&this->m_sound_bank[i], s, f);
		}
	}
}

void
n80_directx::sound_off () {
	this->appendPCMTick(&this->m_sound_bank[0], 0, 0);
	this->appendPCMTick(&this->m_sound_bank[1], 0, 0);
	this->appendPCMTick(&this->m_sound_bank[2], 0, 0);
}
/**--------------------------------------------------------------------------
 **		PC-8001 Speaker ON/OFF
 **------------------------------------------------------------------------*/
void
n80_directx::stop() {
	if (m_dssb2 != NULL)
		m_dssb2->Stop();
	else
		assay->outputLog(TEXT("DirectX:  ds->stop() null reference"));
}

void
n80_directx::play() {
	if (m_dssb2 != NULL)
		m_dssb2->Play(0, 0xffffffff, DSBPLAY_LOOPING);
	else
		assay->outputLog(TEXT("DirectX:  ds->play() null reference"));
}
/**--------------------------------------------------------------------------
 **		Beep/PCG Sound Play
 **------------------------------------------------------------------------*/
LPCBYTE
n80_directx::makepcm() {
	int beep_bank, sound_bank;
	exchangeBank(&beep_bank, &sound_bank);
	return makePCM(beep_bank, sound_bank);
}
void
n80_directx::feedpcm(LPCBYTE pcm) {
	if (!m_mute && pcm != NULL) {
		transferPCM(pcm);
		pcm = NULL;
	} else {
		::ZeroMemory(m_delay_buff, DELAYBUFSIZE);
		transferPCM((LPBYTE)m_delay_buff);
	}
}
/**--------------------------------------------------------------------------
 **		PC-8001 Beep Interval
 **------------------------------------------------------------------------*/
void
n80_directx::appendPCMTick(pSampleTsBank bank, DWORD new_state, ULONG new_freq) {
	pSapmleTsArr arr = &bank->mBanks[bank->mBank];
	if (arr->mCount < MAXTICKARRAY - 1) {
		arr->mArr[arr->mCount].mTicks = bank->mCurrent.mTicks;
		arr->mArr[arr->mCount].mState = bank->mCurrent.mState;
		arr->mArr[arr->mCount].mFreq = bank->mCurrent.mFreq;
		arr->mTotal += bank->mCurrent.mTicks;
		++arr->mCount;
	} else
		assay->outputLog(TEXT("appendPCMTick: bank overrun"));

	bank->mCurrent.mTicks = 0;
	bank->mCurrent.mState = new_state;
	bank->mCurrent.mFreq = new_freq;
}

void
n80_directx::advanceTick(pSampleTsBank bank, LONGLONG diff) {
	LONGLONG add = 0;
	if (bank->mCurrent.mTicks > diff) {
		bank->mCurrent.mTicks -= diff;
		add = diff;
	} else {
		add = bank->mCurrent.mTicks;
		bank->mCurrent.mTicks = 0;
	}
	if (add > 0) {
		pSapmleTsArr arr = &bank->mBanks[bank->mBank];
		if (arr->mCount < MAXTICKARRAY - 1) {
			arr->mArr[arr->mCount].mTicks = add;
			arr->mArr[arr->mCount].mState = bank->mCurrent.mState;
			arr->mArr[arr->mCount].mFreq = bank->mCurrent.mFreq;
			arr->mTotal += add;
			++arr->mCount;
		} else if (arr->mArr[MAXTICKARRAY - 1].mState == bank->mCurrent.mState) {
			arr->mArr[MAXTICKARRAY - 1].mTicks += add;
			arr->mTotal += add;
		} else
			assay->outputLog(TEXT("advanceTick: bank overrun"));
	}
}
/**--------------------------------------------------------------------------
 **		PCM Sound (Sound Thread)
 **------------------------------------------------------------------------*/
static inline void addTk(pSapmleTsArr arr, LONGLONG ticks, DWORD state) {
	arr->mArr[arr->mCount].mTicks = ticks;
	arr->mArr[arr->mCount].mState = state;
	++arr->mCount;
	arr->mTotal += ticks;
}
static inline void appendTk(pSapmleTsArr arr, LONGLONG ticks, DWORD state) {
	if (arr->mCount > 0 && arr->mArr[arr->mCount-1].mState == state)
		arr->mArr[arr->mCount-1].mTicks += ticks;
	else {
		arr->mArr[arr->mCount].mTicks = ticks;
		arr->mArr[arr->mCount].mState = state;
		++arr->mCount;
	}
	arr->mTotal += ticks;
}
void
n80_directx::exchangeBank(int *beep_bank, int *sound_bank) {
	int beep_primary_bank, sound_primary_bank;

	//bank؂ւ
	beep_primary_bank	= this->m_beep_bank.mBank;
	sound_primary_bank	= this->m_sound_bank[0].mBank;
	int beep_sub_bank	= beep_primary_bank == 0 ? 1 : 0;
	int sound_sub_bank	= sound_primary_bank == 0 ? 1 : 0;
	// Sub-BankNA
	this->m_beep_bank.mBanks[beep_sub_bank].mCount = 0;
	this->m_beep_bank.mBanks[beep_sub_bank].mTotal = 0;
	this->m_sound_bank[0].mBanks[sound_sub_bank].mCount = 0;
	this->m_sound_bank[0].mBanks[sound_sub_bank].mTotal = 0;
	this->m_sound_bank[1].mBanks[sound_sub_bank].mCount = 0;
	this->m_sound_bank[1].mBanks[sound_sub_bank].mTotal = 0;
	this->m_sound_bank[2].mBanks[sound_sub_bank].mCount = 0;
	this->m_sound_bank[2].mBanks[sound_sub_bank].mTotal = 0;

	// sݒl[(Beep)
	pSapmleTsArr arr1 = &this->m_beep_bank.mBanks[beep_primary_bank];
	pSapmleTsArr arr2 = &this->m_beep_bank.mBanks[beep_sub_bank];
	if (arr1->mTotal < this->m_tick_delay) {
		LONGLONG diff = this->m_tick_delay - arr1->mTotal;
		this->advanceTick(&m_beep_bank, diff);
	}
	// ߕTuɌJz(Beep)
	if (arr1->mTotal > this->m_tick_delay) {
		pSapmleTsArr _arr = &this->m_beep_copy_arr;
		_arr->mCount = 0;
		_arr->mTotal = 0;
		bool overflow = false;
		for (int i = 0; i < arr1->mCount; ++i) {
			LONGLONG ticks = arr1->mArr[i].mTicks;
			DWORD    state = arr1->mArr[i].mState;
			if (!overflow) {
				LONGLONG frames = _arr->mTotal + ticks;
				if (frames > this->m_tick_delay) {
					LONGLONG over = frames - this->m_tick_delay;
					ticks -= over;

					appendTk(arr2, over, state);
					overflow = true;
				}
				appendTk(_arr, ticks, state);
			} else {
				appendTk(arr2, ticks, state);
			}
		}
		beep_primary_bank = -1;
	}
	//--------- PCG ----------
	pSapmleTsArr arr10 = &this->m_sound_bank[0].mBanks[sound_primary_bank];
	pSapmleTsArr arr20 = &this->m_sound_bank[0].mBanks[sound_sub_bank];
	pSapmleTsArr arr11 = &this->m_sound_bank[1].mBanks[sound_primary_bank];
	pSapmleTsArr arr21 = &this->m_sound_bank[1].mBanks[sound_sub_bank];
	pSapmleTsArr arr12 = &this->m_sound_bank[2].mBanks[sound_primary_bank];
	pSapmleTsArr arr22 = &this->m_sound_bank[2].mBanks[sound_sub_bank];

	// sݒl[(PCG)
	if (arr10->mTotal < this->m_tick_delay) {
		LONGLONG diff = this->m_tick_delay - arr10->mTotal;
		this->advanceTick(&this->m_sound_bank[0], diff);
		this->advanceTick(&this->m_sound_bank[1], diff);
		this->advanceTick(&this->m_sound_bank[2], diff);
	}
	// ߕTuɌJz(PCG)
	if (arr10->mTotal > this->m_tick_delay) {
		pSapmleTsArr _arr0 = &this->m_sound_copy_bank[0];
		pSapmleTsArr _arr1 = &this->m_sound_copy_bank[1];
		pSapmleTsArr _arr2 = &this->m_sound_copy_bank[2];
		_arr0->mCount = 0;
		_arr0->mTotal = 0;
		_arr1->mCount = 0;
		_arr1->mTotal = 0;
		_arr2->mCount = 0;
		_arr2->mTotal = 0;
		volatile bool overflow = false;
		for (int i = 0; i < arr10->mCount; ++i) {
			LONGLONG ticks  = arr10->mArr[i].mTicks;
			DWORD    state0 = arr10->mArr[i].mState;
			DWORD    state1 = arr11->mArr[i].mState;
			DWORD    state2 = arr12->mArr[i].mState;
			if (!overflow) {
				LONGLONG frames = _arr0->mTotal + ticks;
				if (frames > this->m_tick_delay) {
					LONGLONG over = frames - this->m_tick_delay;
					ticks -= over;

					addTk(arr20, over, state0);
					addTk(arr21, over, state1);
					addTk(arr22, over, state2);
					overflow = true;
				}
				addTk(_arr0, ticks, state0);
				addTk(_arr1, ticks, state1);
				addTk(_arr2, ticks, state2);
			} else {
				addTk(arr20, ticks, state0);
				addTk(arr21, ticks, state1);
				addTk(arr22, ticks, state2);
			}
		}
		sound_primary_bank = -1;
	}
	//if (this->m_tick_delay != arr1->mTotal) {
	//	assay->outputLog(TEXT("[U:makePCM beep = %g(), pcg = %g()"),
	//		((double)this->m_tick_delay / (double)(beep_primary_bank == -1 ? this->m_beep_copy_arr.mTotal : this->m_beep_bank.mBanks[beep_primary_bank].mTotal)) * 100,
	//		((double)this->m_tick_delay / (double)this->m_sound_copy_bank[1].mTotal) * 100);
	//}

	// Bank؂ւ
	this->m_beep_bank.mBank = beep_sub_bank;
	for (int i = 0; i < 3; ++i) {
		this->m_sound_bank[i].mBank = sound_sub_bank;
	}

		*beep_bank  = beep_primary_bank;
		*sound_bank = sound_primary_bank;
}

LPCBYTE
n80_directx::makePCM(int beep_primary_bank, int sound_primary_bank) {
	// PCM쐬
	makeNormalPCM(beep_primary_bank, sound_primary_bank);

	return (LPBYTE) m_delay_buff;
}

void
n80_directx::makeNormalPCM(int beep_bank, int sound_bank) {
	::ZeroMemory(m_delay_buff, DELAYBUFSIZE);
	/**----------------------------------------------------------------------
	 **		Beep Wave
	 **--------------------------------------------------------------------*/
	pSapmleTsArr arr = beep_bank == -1 ? &this->m_beep_copy_arr : &this->m_beep_bank.mBanks[beep_bank];
	if (arr->mCount > 0) {
		LONGLONG sum = 0;
		int raster = 0;
		double samples_per_tick = ((double)DELAYSAMPLES) / ((double) arr->mTotal);
		for (int i = 0; i < (int) arr->mCount; ++i) {
			//int samples = (int) floor(((double) arr->mArr[i].mTicks) * samples_per_tick + 0.5);
			sum += arr->mArr[i].mTicks;
			int index2 = (int) floor(((double) sum) * samples_per_tick + 0.5);
			int samples = index2 - raster;
			if (samples > 0) {
				for (int j = 0; j < samples; ++j) {
					int k = raster + j;
					if (k >= DELAYSAMPLES)
						break;
					else if (arr->mArr[i].mState == 0)
						;
					else {
						// ʑ𑵂
						int phase = this->m_beep_phase_adjuster % SMPL2400;
						if (phase < (SMPL2400/2))
							m_delay_buff[k] += m_beep_amplitude;
						else
							m_delay_buff[k] -= m_beep_amplitude;
					}
					if (++this->m_beep_phase_adjuster >= 10000000)
						this->m_beep_phase_adjuster = 0;
				}
			}
			raster += samples;
		}
		// s𖄂߂
		//if (index < (SMPLRATE/50)) {
		//	if (arr->mArr[arr->mCount - 1].mState == 1) {
		//		for (int i = index; i < (SMPLRATE/50); ++i) {
		//			// ʑ𑵂
		//			int phase = this->m_beep_phase_adjuster % SMPL2400;
		//			if (phase < (SMPL2400/2))
		//				m_delay_buff[i] += m_beep_amplitude;
		//			else
		//				m_delay_buff[i] -= m_beep_amplitude;
		//			if (++this->m_beep_phase_adjuster >= 10000000)
		//				this->m_beep_phase_adjuster = 0;
		//		}
		//	}
		//}
	} else {
		assay->outputLog(TEXT("makeNormalPCM: no data output"));
	}
	/**----------------------------------------------------------------------
	 **		PCG Wave+
	 **--------------------------------------------------------------------*/
	pSapmleTsArr	arr0 = sound_bank == -1 ? &this->m_sound_copy_bank[0] : &this->m_sound_bank[0].mBanks[sound_bank];
	pSapmleTsArr	arr1 = sound_bank == -1 ? &this->m_sound_copy_bank[1] : &this->m_sound_bank[1].mBanks[sound_bank];
	pSapmleTsArr	arr2 = sound_bank == -1 ? &this->m_sound_copy_bank[2] : &this->m_sound_bank[2].mBanks[sound_bank];

	if (arr0->mCount > 0) {
		LONGLONG sum = 0;
		int raster = 0, total = 0, t0 = 0, thalf0 = 0, t1 = 0, thalf1 = 0, t2 = 0, thalf2 = 0, count_zero = 0;

		double samples_per_tick = ((double)DELAYSAMPLES) / ((double) arr0->mTotal);
		for (int i = 0; i < (int) arr0->mCount; ++i) {
			sum += arr0->mArr[i].mTicks;
			int index2 = (int) floor(((double) sum) * samples_per_tick + 0.5);
			int samples = index2 - raster;
			if (samples > 0) {
				//Tick̗݌v狁߂Index
				t0 = arr0->mArr[i].mState;
				thalf0 = t0 / 2;
				t1 = arr1->mArr[i].mState;
				thalf1 = t1 / 2;
				t2 = arr2->mArr[i].mState;
				thalf2 = t2 / 2;
				for (int j = 0; j < samples; ++j) {
					int k = raster + j;
					if (k >= DELAYSAMPLES)
						break;
					int volume = 0;
					if (/*n80device.hal.speaker[0] == 1 &&*/ t0 > 0) {
						int phase = this->m_sound_phase_adjuster % t0;
						if (phase < thalf0)
							volume += m_pcg_amplitude;
						else
							volume -= m_pcg_amplitude;
					}
					if (/*n80device.hal.speaker[1] == 1 &&*/ t1 > 0) {
						int phase = this->m_sound_phase_adjuster % t1;
						if (phase < thalf1)
							volume += m_pcg_amplitude;
						else
							volume -= m_pcg_amplitude;
					}
					if (/*n80device.hal.speaker[2] == 1 &&*/ t2 > 0) {
						int phase = this->m_sound_phase_adjuster % t2;
						if (phase < thalf2)
							volume += m_pcg_amplitude;
						else
							volume -= m_pcg_amplitude;
					}
					volume += m_delay_buff[k];
					if (volume > 32767/*this->m_beep_amplitude*/)
						volume = 32767;//this->m_beep_amplitude;
					else if (volume < -32768/*((int)this->m_beep_amplitude * -1)*/)
						volume = -32768;//this->m_beep_amplitude * -1;
					m_delay_buff[k] = (short)volume;
					if (++this->m_sound_phase_adjuster >= 100000000)
						this->m_sound_phase_adjuster = 0;
				}
			}
			raster += samples;
		}
		//assay->outputLog(TEXT("[U:makePCM pcg[%d] = %g()"), ch, ((double)index / (double)(SMPLRATE/10)) * 100);
		 //s𖄂߂
		//if (index < (SMPLRATE/10)) {
		//	for (int i = index; i < (SMPLRATE/10); ++i) {
		//		int volume = 0;
		//		if (/*n80device.hal.speaker[ch] == 1 &&*/ t > 0) {
		//			int phase = this->m_sound_phase_adjuster[ch] % t;
		//			if (phase < thalf)
		//				volume = m_delay_buff[i] + (m_pcg_amplitude / 3);
		//			else
		//				volume = m_delay_buff[i] - (m_pcg_amplitude / 3);

		//			if (volume > this->m_beep_amplitude)
		//				volume = this->m_beep_amplitude;
		//			else if (volume < ((int)this->m_beep_amplitude * -1))
		//				volume = this->m_beep_amplitude * -1;
		//			m_delay_buff[i] = (short)volume;
		//		}
		//		if (++this->m_sound_phase_adjuster[ch] >= 100000000)
		//			this->m_sound_phase_adjuster[ch] = 0;
		//	}
		//}
	} else {
		assay->outputLog(TEXT("makeNormalPCM: no data output"));
	}
}
/**--------------------------------------------------------------------------
 **		PCM Data transfer (Sound Thread)
 **------------------------------------------------------------------------*/
void
n80_directx::transferPCM(LPCBYTE pcm) {
	LPVOID	lpSoundData1, lpSoundData2;
	DWORD	size1, size2, play, pos;
	
	if (SUCCEEDED(m_dssb2->GetCurrentPosition(&play, &pos))) {
		//DWORD next_sector = ((play / DELAYBUFSIZE) + 1) % 4;
		DWORD next_sector = this->next_sector(pos / DELAYBUFSIZE);
		if (m_next_sector != next_sector) {
			if (next_sector == this->prev_sector(m_next_sector))
				next_sector = m_next_sector;
		}
		m_next_sector = this->next_sector(next_sector);
		DWORD offset = next_sector * DELAYBUFSIZE;
		//DWORD offset = pos;
		if (SUCCEEDED(m_dssb2->Lock(offset, DELAYBUFSIZE,
						&lpSoundData1, &size1,
						&lpSoundData2, &size2,
						0))) {
						//DSBLOCK_FROMWRITECURSOR))) {
						//DSBLOCK_ENTIREBUFFER))) {
			::CopyMemory(lpSoundData1, pcm, size1);
			if (lpSoundData2 != NULL && size2 > 0) {
				::CopyMemory(lpSoundData2, &pcm[size1], size2);
			}
			m_dssb2->Unlock(lpSoundData1, size1, lpSoundData2, size2);
		} else
			assay->outputLog(TEXT("transferPCM: fail to Lock pcm buffer"));
	}
	return;
#if defined(_TEST) || defined(_DEBUG)
	// tԊu
	assay->takeFeedIntervals();
#endif
}
