/**--------------------------------------------------------------------------
 **
 **		NEC PC-8001 Emulator "N80" for Win32/DirectX
 **		--------------------------------------------
 **		Copyright (c) 1993-2010 H.Kanemoto
 **
 **		Initialize N80
 **
 **		Rev		Date		Descriptions
 **		-----	----------	-----------------------------------------------
 **		1.00	1996/06/27	porting DOS version on WinG
 **		1.20	1997/08/10	porting WinG version on Win32/DirectX
 **		1.30	1998/03/12	
 **		1.43	2007/09/01	XPΉ
 **		1.44	2009/12/15	Vista/7Ή
 **		1.51	2010/01/27	DirectSound OnlyɕύX
 **------------------------------------------------------------------------*/
#include "n80_form.h"
#include "n80_directx.h"
#include "../z80/z80_thread.h"
#include "../lib/icj_stream.h"
#include "../lib/icj_memory.h"
#include "../res/resource.h"
/**--------------------------------------------------------------------------
 **		O[oϐϐ
 **------------------------------------------------------------------------*/
extern LPCTSTR		cAPPNAME;
extern n80_directx	*dx;
extern z80_thread	*th;
extern n80_assay	*assay;
extern "C" {
extern void __cdecl	_n80_device_initialize(LPN80DEVICE, SYNCFNC enter, SYNCFNC leave);

static LPCRITICAL_SECTION csec;
static void enter(void) {
	::EnterCriticalSection(csec);
}

static void leave(void) {
	::LeaveCriticalSection(csec);
}

static void _n80_beep_on() {
	dx->beep_on();
}

static void _n80_beep_off() {
	dx->beep_off();
}

static void _n80_sound_on(USHORT freq, int ch) {
	dx->sound_on(freq, ch);
}

static void _n80_sound_off() {
	dx->sound_off();
}

static void _n80_append_sound_ticks(DWORD T) {
	dx->appendTs((LONGLONG)T);
}

static void _n80_clear_pcg_ticks() {
	dx->clear_Ts_bank();
}

static LONG _n80win_get_hard_volume(void) {
	return dx->getHardVolume();
}

static void _n80win_set_hard_volume(LONG v) {
	dx->setHardVolume(v);
}

static SHORT _n80win_get_soft_volume(void) {
	return dx->getSoftVolume();
}

static void _n80win_set_soft_volume(SHORT v) {
	dx->setSoftVolume(v);
}

export void _n80_init_snd(void) {
	n80device.snd.beep_on		= _n80_beep_on;
	n80device.snd.beep_off		= _n80_beep_off;
	n80device.snd.sound_on		= _n80_sound_on;
	n80device.snd.sound_off		= _n80_sound_off;
	n80device.snd.append_ticks	= _n80_append_sound_ticks;
	n80device.snd.clear_ticks	= _n80_clear_pcg_ticks;

	n80win32.set_hard_volume	= _n80win_set_hard_volume;
	n80win32.get_hard_volume	= _n80win_get_hard_volume;
	n80win32.set_soft_volume	= _n80win_set_soft_volume;
	n80win32.get_soft_volume	= _n80win_get_soft_volume;
}
}// extern "C"
/**--------------------------------------------------------------------------
 **		N80
 **------------------------------------------------------------------------*/
BOOL
n80_form::init() {
	assay->outputLog(TEXT("IME֎~"));
	// IME֎~
	::ImmAssociateContext(mWnd, NULL);

	memset(&n80win32, 0, sizeof(N80));

	// PC-8001 Device Initialize
	csec = &m_csec;
	_n80_device_initialize(&n80device, enter, leave);

	memset(&n80win32.n80_rom_file_name, 0, sizeof(n80win32.n80_rom_file_name));
	memset(&n80win32._4th_rom_file_name, 0, sizeof(n80win32._4th_rom_file_name));
	memset(&n80win32.fdu_rom_file_name, 0, sizeof(n80win32.fdu_rom_file_name));

	// INIt@C
	if (!initProfile()) {
		::MessageBox(NULL, TEXT("INIt@Cǂݍݎs"), cAPPNAME, MB_ICONSTOP | MB_OK);
		return FALSE;
	}
	// ROMC[Wǂݍ
	if (!loadROM()) {
		::MessageBox(NULL, TEXT("ROMC[Wt@Cǂݍݎs"), cAPPNAME, MB_ICONSTOP | MB_OK);
		return FALSE;
	}
	// DISK ROMC[Wǂݍ
	n80device.fdu.set_rom_ready(&n80device.fdu, loadDiskROM());

	// BMPC[Wǂݍ
	if (!loadBMP()) {
		::MessageBox(NULL, TEXT("BMP\[Xǂݍݎs"), cAPPNAME, MB_ICONSTOP | MB_OK);
		return FALSE;
	}
	// ICONC[Wǂݍ
	if (!loadIcon()) {
		::MessageBox(NULL, TEXT("ICON\[Xǂݍݎs"), cAPPNAME, MB_ICONSTOP | MB_OK);
		return FALSE;
	}
	// FONTC[Wǂݍ
	if (!loadFont()) {
		::MessageBox(NULL, TEXT("FNT\[Xǂݍݎs"), cAPPNAME, MB_ICONSTOP | MB_OK);
		return FALSE;
	}

	n80device.blt.chg_palette(n80device.param.interlace);
	clearRAM();		// Z80 RAM Area Clear
	initKeyb();		// init PC-8001 KeyBoard
	//_blt_mmx();		// Use MMX instractions

	// IuWFNg̐
	m_hevt_sync = ::CreateEvent(NULL, FALSE, FALSE, "SYNC");
	m_hevt_ack  = ::CreateEvent(NULL, FALSE, FALSE, "ACK");

	// Z80Xbh
	if (!th->init()) {
		assay->outputLog(TEXT("Z80Xbhs"));
		return FALSE;
	}
	// DirectX
	if (!dx->init_dx(mWnd)) {
		assay->outputLog(TEXT("DirectXs"));
		//::MessageBox(NULL, TEXT("DirectXs"), cAPPNAME, MB_ICONSTOP | MB_OK);
		return FALSE;
	}
	dx->setHardVolume(n80win32.hard_volume);
	dx->setSoftVolume((short)n80win32.soft_volume);

	// Z80/Sound ThreadJnCxg|Xg
	postMessage(WM_Z80_START, 0, 0);

	return TRUE;
}
/**--------------------------------------------------------------------------
 **		Z80 RAM Area Clear
 **------------------------------------------------------------------------*/
void
n80_form::clearRAM() {
	n80device.mem.clear();
}
/**--------------------------------------------------------------------------
 **		ROMt@Cǂݍ
 **------------------------------------------------------------------------*/
BOOL
n80_form::loadROM() {
	icj_fstream	rom;
	DWORD		fsiz, rsiz;

	if (!rom.open(n80win32.n80_rom_file_name, TRUE))
		return  FALSE;
	fsiz = rom.getSize();
	if (fsiz > 32*1024)
		return  FALSE;
	// 8801N-BASIC ROMȂǁA24KBްROMǂݍ񂾏ꍇ4th ROMƂ݂Ȃ
	n80device.param.attach_4th_rom = fsiz > (24 * 1024);
	rsiz = rom.read((LPVOID) n80device.mem._rom, fsiz, 1);
	if (fsiz != rsiz)
		return  FALSE;
	rom.close();

	// 4th ROM̎w肪ꍇ̂݃IvVœǂݍ
	if (!n80device.param.attach_4th_rom && lstrlen(n80win32._4th_rom_file_name) > 0) {
		if (rom.open(n80win32._4th_rom_file_name, TRUE)) {
			fsiz = rom.getSize();
			if (fsiz <= 8 * 1024) {	// 4th ROM8KBȉ
				rsiz = rom.read((LPVOID)n80device.mem._4th, fsiz, 1);
				if (fsiz != rsiz)
					return  FALSE;
			}
		}
	}
	return  TRUE;
}
/**--------------------------------------------------------------------------
 **		ROMt@Cǂݍ
 **------------------------------------------------------------------------*/
BOOL
n80_form::loadDiskROM() {
	icj_fstream	rom;
	DWORD		fsiz, rsiz;

	if (lstrlen(n80win32.fdu_rom_file_name) == 0)
		return FALSE;
	if (!rom.open(n80win32.fdu_rom_file_name, TRUE))
		return  FALSE;
	fsiz = rom.getSize();
	if (fsiz > 8 * 1024)
		return  FALSE;
	rsiz = rom.read((LPVOID)n80device.fdu.mem._rom, fsiz, 1);
	if (fsiz != rsiz)
		return  FALSE;
	rom.close();
	return  TRUE;
}
/**--------------------------------------------------------------------------
 **		DIBC[Wǂݍ݁i\[X)
 **------------------------------------------------------------------------*/
BOOL
n80_form::loadBMP() {
	if (!mLogo.attach((WORD) IDB_BITMAP1))
		return  FALSE;
	mPal = ::CreatePalette(mLogo.getLogPal());
	return  TRUE;
}
/**--------------------------------------------------------------------------
 **		ICONC[Wǂݍ݁i\[X)
 **------------------------------------------------------------------------*/
BOOL
n80_form::loadIcon() {
	mIcon = ::LoadIcon(mInstance, MAKEINTRESOURCE(IDI_ICON1));
	return  mIcon != NULL;
}
/**--------------------------------------------------------------------------
 **		FONTC[Wǂݍ݁i\[X)
 **------------------------------------------------------------------------*/
BOOL
n80_form::loadFont() {
	HGLOBAL	fn;
	HRSRC	rc;
	LPVOID	fd;

	rc = ::FindResource(NULL, MAKEINTRESOURCE(IDF_FONT1), "FNT");
	if (rc != NULL) {
		fn = ::LoadResource(NULL, rc);
		if (fn != 0) {
			fd = (LPVOID) ::LockResource(fn);
			if (fd != NULL) {
				::CopyMemory(n80device.font, fd, 4096);
				::CopyMemory(n80device.pcg, fd, 2048);
				return  TRUE;
			}
		}
	}
	return FALSE;
}
/**--------------------------------------------------------------------------
 **		WindowӰޏ
 **------------------------------------------------------------------------*/
void
n80_form::prepareWindowMode(int width, int height) {
	// Window StyleύX
	DWORD dwStyle = ::GetWindowLong(mWnd, GWL_STYLE);
	dwStyle |= WS_POPUP | WS_SYSMENU | WS_CAPTION;
	if (::SetWindowLong(mWnd, GWL_STYLE, dwStyle) == 0)
		assay->outputLog(TEXT("DirectX:  SetWindowLong Failed"));

	// Window Sizeݒ(Clienẗ = 640x400)
	RECT rc = {0, 0, width, height};
	::AdjustWindowRectEx(&rc, ::GetWindowLong(mWnd, GWL_STYLE), FALSE, 0);
	::SetWindowPos(mWnd, HWND_NOTOPMOST, 0, 0, rc.right-rc.left, rc.bottom-rc.top, SWP_NOMOVE|SWP_NOACTIVATE);

	// ʒ\
	RECT rcDesk, rcWnd;
	HWND hWndDesk = ::GetDesktopWindow();
	::GetWindowRect(hWndDesk, &rcDesk);
	::GetWindowRect(mWnd, &rcWnd);

	int x = rcDesk.right /2 - rcWnd.right /2;
	int y = rcDesk.bottom/2 - rcWnd.bottom/2;
	::SetWindowPos(mWnd, HWND_TOP, x, y, 0, 0, SWP_NOSIZE);
	::ShowWindow(mWnd, SW_SHOW);

	//VXej[
	HMENU sysMenu = ::GetSystemMenu(mWnd , FALSE);
	//::DeleteMenu(sysMenu, 5, MF_BYPOSITION);
	::DeleteMenu(sysMenu, SC_RESTORE, MF_BYCOMMAND);
	//::DeleteMenu(sysMenu, SC_MOVE, MF_BYCOMMAND);
	::DeleteMenu(sysMenu, SC_SIZE , MF_BYCOMMAND);
	::DeleteMenu(sysMenu, SC_MINIMIZE, MF_BYCOMMAND);
	::DeleteMenu(sysMenu, SC_MAXIMIZE, MF_BYCOMMAND);
	//::DeleteMenu(sysMenu, SC_CLOSE, MF_BYCOMMAND);
	::DrawMenuBar(mWnd);

	//N80pj[ǉ
	MENUITEMINFO mii;
	mii.cbSize = sizeof(MENUITEMINFO);
	mii.fMask  = MIIM_STATE;
	if (!::GetMenuItemInfo(sysMenu, IDM_N80HELP, FALSE, &mii)) {
		::ZeroMemory(&mii, sizeof(MENUITEMINFO));
		mii.cbSize 		= sizeof(MENUITEMINFO);
		mii.fMask 		= MIIM_TYPE | MIIM_ID;
		mii.wID 		= IDM_N80HELP;
		mii.fType 		= MFT_STRING;
		mii.dwTypeData 	= TEXT("N80ɂ");
		::InsertMenuItem(sysMenu , 0 , TRUE , &mii);

		::ZeroMemory(&mii, sizeof(MENUITEMINFO));
		mii.cbSize = sizeof(MENUITEMINFO);
		mii.fMask = MIIM_TYPE | MIIM_ID;
		mii.wID = IDM_N80RESET;
		mii.fType = MFT_STRING;
		mii.dwTypeData = TEXT("Zbg(&F12)");
		::InsertMenuItem(sysMenu, 1, TRUE, &mii);

		//::ZeroMemory(&mii, sizeof(MENUITEMINFO));
		//mii.cbSize 		= sizeof(MENUITEMINFO);
		//mii.fMask 		= MIIM_TYPE | MIIM_ID;
		//mii.wID 		= IDM_N80CONFIG;
		//mii.fType 		= MFT_STRING;
		//mii.dwTypeData 	= TEXT("N80̐ݒ");
		//::InsertMenuItem(sysMenu , 1 , TRUE , &mii);
	}
}
/**--------------------------------------------------------------------------
 **		FullScreenӰޏ
 **------------------------------------------------------------------------*/
void
n80_form::prepareFullScreenMode(int width, int height) {
	// Window StyleύX
	DWORD dwStyle = ::GetWindowLong(mWnd, GWL_STYLE);
	dwStyle &= ~(WS_POPUP|WS_CAPTION|WS_SYSMENU);
	::SetWindowLong(mWnd, GWL_STYLE, dwStyle);
}