/**==========================================================================

	PC-8001 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2010 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	External assembler functions for IBM-PC with VC++ 5.0
	last modified  Apr,4,1998.

;==========================================================================*/
#include "PC-8001.h"
#include <stdio.h>
#ifdef _MSC_VER
#include <windows.h>
#endif
#ifdef __cplusplus
extern "C"
{
#endif
#define export

#define	_NONE	(0)
#define	_OCW1	(1)
#define	_OCW5	(5)
#define _OCW1_DM (0x01)	//0:normal, 1:reverse
/**--------------------------------------------------------------------------
 **		ASM Modules
 **------------------------------------------------------------------------*/
static void _n80_crtc_cmd(BYTE cmd) {
	switch ((cmd & 0xe0) >> 5) {
	case 0:	// icw/ocw1 (reset/stop display)
			n80device.crtc._prev_cmd = _OCW1;
			n80device.crtc._param_count = 0;
			n80device.crtc._text_visible |= _TXTVIS_STOPDISP;
			n80device.dmac._penarty = 0;
			n80device.vrtc._dma_remain_chars = 0;
#if _DEBUG
			n80device.logger("CRTC: [OCW1] Reset/Stop Display\n");
#endif
			break;
	case 1:	// ocw2 (start display)
			n80device.crtc._prev_cmd = _NONE;
			n80device.crtc._param_count = 0;
			n80device.crtc._attr_reverse = cmd & _OCW1_DM ? 1 : 0;
			n80device.attr.invalid();
			n80device.crtc._text_visible &= ~_TXTVIS_STOPDISP;
#if _DEBUG
			n80device.logger("CRTC: [OCW2] Start Display (DM:%d)\n", n80device.crtc._attr_reverse);
#endif
			break;
	case 4:	// ocw5 (load cursor position)
			n80device.crtc._prev_cmd = _OCW5;
			n80device.crtc._param_count = 0;
			if (cmd & 0x01)
				n80device.cursor.enable();
			else
				n80device.cursor.disable();
#if _DEBUG
			//n80device.logger("CRTC: [OCW5] Load Cursor Position\n");
#endif
			break;

	case 2:	// ocw3 (set interrupt mask)
#if _DEBUG
		n80device.logger("CRTC: [OCW3] Set Interrupt Mask\n");
#endif
			break;
	case 3:	// ocw4 (read light pen)
#if _DEBUG
		n80device.logger("CRTC: [OCW4] Read Lignt Pen\n");
#endif
			break;
	case 5:	// ocw6 (reset interrupt)
#if _DEBUG
		n80device.logger("CRTC: [OCW6] Reset Interrupt\n");
#endif
			break;
	case 6:	// ocw7 (reset counters)
#if _DEBUG
		n80device.logger("CRTC: [OCW7] Reset Counters\n");
#endif
			break;
	case 7:	// ocw8
	default:
			n80device.crtc._prev_cmd = _NONE;
			n80device.crtc._param_count = 0;
			break;
	}
}

static void _n80_crtc_data(BYTE dat) {
	USHORT	cnt = n80device.crtc._param_count++;
	BYTE	al, ah;

	switch (n80device.crtc._prev_cmd) {
	case _OCW1:	
		switch (cnt) {
		case 0:	//screen format 1
			n80device.crtc._burst_mode = (dat & 0x80) == 0 ? 1 : 0;
			n80device.crtc._chars_per_line = (dat & 0x7f) + 2;
#if _DEBUG
			n80device.logger("CRTC: DMA burst:\'%s\',\nCRTC - width:%d, ", n80device.crtc._burst_mode == 1 ? "B" : "C", n80device.crtc._chars_per_line);
#endif
			break;
		case 1:	//screen format 2
			ah = (dat & 0x3f) + 1;
			al = ah == 25 ? 1 : 0;
			n80device.crtc._lines_per_screen = ah;
#if _DEBUG
			n80device.logger("CRTC: lines=%d, ", n80device.crtc._lines_per_screen);
#endif
			if (n80device.crtc._line_mode != al) {
				n80device.crtc._line_mode = al;
				if (n80device.crtc._width_mode == 1)
					al += 0x02;
				n80device.crtc._screen_mode = al;
				n80device.attr.invalid();
				n80device.vrtc.reset();	// change crtc sync cycle
				n80device.blt.chg_crt_mode(al);
			}
			//cursor/attribute blinking rate
			ah = ((dat & 0xc0) >> 6);
			switch (ah) {
			case 0:
				n80device.cursor._cur_blink_cycl = 16;
				n80device.cursor._cur_blink_disp = 8;
				n80device.cursor._att_blink_cycl = 32;
				n80device.cursor._att_blink_disp = 24;
				break;
			case 1:
				n80device.cursor._cur_blink_cycl = 32;
				n80device.cursor._cur_blink_disp = 16;
				n80device.cursor._att_blink_cycl = 64;
				n80device.cursor._att_blink_disp = 48;
				break;
			case 3:
				n80device.cursor._cur_blink_cycl = 64;
				n80device.cursor._cur_blink_disp = 32;
				n80device.cursor._att_blink_cycl = 128;
				n80device.cursor._att_blink_disp = 96;
				break;
			default:
				n80device.cursor._cur_blink_cycl = 48;
				n80device.cursor._cur_blink_disp = 24;
				n80device.cursor._att_blink_cycl = 96;
				n80device.cursor._att_blink_disp = 72;
				break;
					}
			break;
		case 2:	//screen format 3
			n80device.crtc._raster_per_char = (dat & 0x1f) + 1;
#if _DEBUG
			n80device.logger("CRTC: Raster/Char: %d, ", n80device.crtc._raster_per_char);
#endif
			break;
		case 3:	//screen format 4
			n80device.crtc._ver_ret_lines = ((dat & 0xe0) >> 5) + 1;
			n80device.crtc._hol_ret_chars = (dat & 0x1f) + 1;
#if _DEBUG
			n80device.logger("CRTC: VRL=%d, HRC=%d\n", n80device.crtc._ver_ret_lines, n80device.crtc._hol_ret_chars);
#endif
			break;
		case 4:	//screen format 5
			ah = ((dat & 0xe0) >> 5);	//AT1/AT0/SC
			switch (ah) {
				case 5/*0b101*/:	// non-transparent mono, special char disabled
					n80device.crtc._attribute_en = 1;
					n80device.crtc._attribute_mono = 1;
					n80device.crtc._attribute_tp = 0;
					n80device.crtc._special_char = 0;
					break;
				case 4/*0b100*/:	// non-transparent mono, special char enabled
					n80device.crtc._attribute_en = 1;
					n80device.crtc._attribute_mono = 1;
					n80device.crtc._attribute_tp = 0;
					n80device.crtc._special_char = 1;
					break;
				case 0/*0b000*/:	// transparent mono, special char enabled
					n80device.crtc._attribute_en = 1;
					n80device.crtc._attribute_mono = 1;
					n80device.crtc._attribute_tp = 1;
					n80device.crtc._special_char = 1;
					break;
				case 2/*0b010*/:	// transparent color, special char enabled
					n80device.crtc._attribute_en = 1;
					n80device.crtc._attribute_mono = 0;
					n80device.crtc._attribute_tp = 1;
					n80device.crtc._special_char = 1;
					break;
				//case 0b001:		// no attribute, special char disabled
				default:
					n80device.crtc._attribute_en = 0;
					n80device.crtc._attribute_mono = 1;	//N/A
					n80device.crtc._attribute_tp = 0;	//N/A
					n80device.crtc._special_char = 0;	//N/A
					break;
			}
			al = (dat & 0x1f) + 1;	//attribute bytes
			n80device.crtc._crtc_line_bytes = n80device.crtc._chars_per_line;
			if (n80device.crtc._attribute_en == 1) {
				n80device.crtc._crtc_line_bytes += (al * 2);
				// in non-transparent mode, special char area follow the char area 
				// in transparent mode, special char area is included at tail of attribute
				if (n80device.crtc._special_char == 1 && n80device.crtc._attribute_tp == 0)
					n80device.crtc._crtc_line_bytes += 2;
			}
#if _DEBUG
			n80device.logger("CRTC - attribute:%s, transparent:%s, color:%s, sp:%s, size:%d, line-size:%d\n",
				n80device.crtc._attribute_en == 1 ? "en" : "dis",
				n80device.crtc._attribute_tp == 1 ? "en" : "dis",
				n80device.crtc._attribute_mono == 1 ? "mono" : "color",
				n80device.crtc._special_char == 1 ? "en" : "dis", al, n80device.crtc._crtc_line_bytes
			);
#endif
			break;
		default:
			n80device.crtc._prev_cmd = _NONE;
			break;
		}
		break;

	case _OCW5:	if (cnt == 0)
					n80device.cursor._new_col = dat & 0x7f;
				else {
					n80device.cursor._new_row = dat & 0x3f;
					n80device.crtc._prev_cmd = _NONE;
				}
				n80device.cursor.move();
				break;
	}
}

static void _n80_width_mode(BYTE dat) {
	BYTE	ah;
	int		changed = 0;

	// ύX`FbN
	ah = dat & 0x01;
	if (n80device.crtc._width_mode != ah) {
		n80device.crtc._width_mode = ah;
		ah <<= 1;
		ah += n80device.crtc._line_mode;
		n80device.crtc._screen_mode = ah;
		n80device.blt.chg_crt_mode(ah);
		changed = 1;
	}
	// J[[hύX`FbN
	ah = (dat & 0x02) >> 1;
	if (n80device.crtc._color_mode != ah) {
		n80device.crtc._color_mode = ah;
		n80device.attr.chg_converter(ah);
		changed = 1;
	}
	if (changed)
		n80device.attr.invalid();
}

static void _n80_reset_crtc(void) {
	n80device.crtc._attr_reverse	= 0;
	n80device.crtc._color_mode		= 1;
	n80device.crtc._line_mode		= 1;
	n80device.crtc._text_visible	= _TXTVIS_STOPDISP;
	n80device.crtc._crtc_line_bytes	= 0;
	n80device.crtc._burst_mode		= 0;	//off
	n80device.crtc._raster_per_char	= 10;
}

export void _n80_init_crtc(void) {
	n80device.crtc.reset	= _n80_reset_crtc;
	n80device.crtc.cmd		= _n80_crtc_cmd;
	n80device.crtc.data		= _n80_crtc_data;
	n80device.crtc.mode		= _n80_width_mode;
	_n80_reset_crtc();
}

#ifdef __cplusplus
}	// extern "C"
#endif
