/**==========================================================================
 
	PC-8001 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2010 H.Kanemoto
 
	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.
 
	External assembler functions for IBM-PC with VC++ 5.0
	last modified  Apr,4,1998.
 
 ;==========================================================================*/
#include "PC-8001.h"
#include <string.h>
#ifdef _MSC_VER
#include <windows.h>
#endif
#ifdef __cplusplus
extern "C"
{
#endif
#define export

#define _EN_DMA_CH2		 (0x04)
#define _AUTOLOAD		 (0x80)
/**--------------------------------------------------------------------------
 **		ASM Modules
 **------------------------------------------------------------------------*/
static void _n80_reset_dmac(void) {
	int ch;
	n80device.dmac._begin = 0xf3c8;
	n80device.dmac._end = 0xff80;
	n80device.dmac._penarty = 0;
	n80device.dmac._dmaact = 0;
	for (ch = 0; ch < 4; ++ ch) {
		memset(&n80device.dmac.ch[ch], 0, sizeof(N80DMAC_CH));
	}
}

static void _n80_dmac_cmd(BYTE v) {
	BYTE en = v & _EN_DMA_CH2;
	int ch;
	//BYTE chg = n80device.dmac._dmaact ^ en;
	if (en)
		n80device.crtc._text_visible &= ~_TXTVIS_STOPDMA;
	else {
		n80device.crtc._text_visible |= _TXTVIS_STOPDMA;
		n80device.dmac._penarty = 0;
	}
	n80device.dmac._dmaact = en;
	n80device.dmac._autoload = (v & _AUTOLOAD) != 0 ? 1 : 0;
	//clear except byte order
	for (ch = 0; ch < 4; ++ ch) {
		n80device.dmac.ch[ch]._addr_expect_hl = 0;
		n80device.dmac.ch[ch]._cnt_expect_hl = 0;
	}
}

static void _n80_dmac_ch_addr(int ch, BYTE v) {
	if (n80device.dmac.ch[ch]._addr_expect_hl == 0) {
		n80device.dmac.ch[ch]._addr_expect_hl = 1;
		n80device.dmac.ch[ch]._addr_lo = v;
	} else {
		USHORT addr;
		n80device.dmac.ch[ch]._addr_expect_hl = 0;
		n80device.dmac.ch[ch]._addr_hi = v;
		addr = ((USHORT)n80device.dmac.ch[ch]._addr_hi << 8) + ((USHORT)n80device.dmac.ch[ch]._addr_lo);
		n80device.dmac.ch[ch]._addr = addr;
		if (ch == 2) {
			n80device.dmac._begin = addr;
#if _DEBUG
			n80device.logger("DMAC: [ch2] begin = %4.4X\n", n80device.dmac._begin);
#endif
			if (n80device.dmac._autoload == 1) {
				n80device.dmac.ch[3]._addr = addr;
			}
		}
#if _DEBUG
		else if (ch == 3) {
			n80device.logger("DMAC: [ch3] begin = %4.4X\n", addr);
		}
#endif
	}
}

static void _n80_dmac_ch_cnt(int ch, BYTE v) {
	if (n80device.dmac.ch[ch]._cnt_expect_hl == 0) {
		n80device.dmac.ch[ch]._cnt_expect_hl = 1;
		n80device.dmac.ch[ch]._cnt_lo = v;
	} else {
		USHORT count;
		n80device.dmac.ch[ch]._cnt_expect_hl = 0;
		n80device.dmac.ch[ch]._cnt_hi = v & 0x3f;
		n80device.dmac.ch[ch]._wr_mode = ((v & 0xc0) >> 6);
		count = ((USHORT)n80device.dmac.ch[ch]._cnt_hi << 8) + ((USHORT)n80device.dmac.ch[ch]._cnt_lo) + 1;
		n80device.dmac.ch[ch]._cnt = count;
		n80device.dmac.ch[ch]._remain = count;
		if (ch == 2) {
			n80device.dmac._end = n80device.dmac._begin + count;
#if _DEBUG
			n80device.logger("DMAC: [ch2] end = %4.4X\n", n80device.dmac._end);
#endif
			if (n80device.dmac._autoload == 1) {
				n80device.dmac.ch[3]._cnt = count;
				n80device.dmac.ch[3]._remain = count;
			}
		}
#if _DEBUG
		else if (ch == 3) {
			n80device.logger("DMAC: [ch3] end = %4.4X\n", n80device.dmac.ch[ch]._addr + count);
		}
#endif
	}
}

export void _n80_init_dmac(void) {
	n80device.dmac.reset	= _n80_reset_dmac;
	n80device.dmac.cmd		= _n80_dmac_cmd;
	n80device.dmac.ch_adr	= _n80_dmac_ch_addr;
	n80device.dmac.ch_cnt	= _n80_dmac_ch_cnt;
	_n80_reset_dmac();
}

#ifdef __cplusplus
}	// extern "C"
#endif
