/**==========================================================================

	PC-8001 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2010 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	External assembler functions for IBM-PC with VC++ 5.0
	last modified  Apr,4,1998.

;==========================================================================*/
#include "PC-8001.h"
#ifdef __APPLE__
#include <string.h>
#endif
#ifdef __cplusplus
extern "C"
{
#endif
#if _DEBUG
	static int send_fdu_command_code = 0;
	static int send_fdu_command_cnt = 0;
	static int send_fdu_command_remain = 0;
	static int recv_fdu_command_remain = 0;
	static int recv_fdu_command_cnt = 0;
#endif
#define export
/**--------------------------------------------------------------------------
 **		ASM Modules
 **------------------------------------------------------------------------*/
static BYTE __cdecl _n80_ioread(LPZ80CPU cpu, USHORT port)
{
	switch ((port & 0xf0)) {
	case 0x00:	if ((port & 0x0f) < 10)
					return n80device.io.keys[port & 0x0f];
				else
					return 0xff;

	case 0x20:	if (port & 0x01)
					return 0x07;
				else
					return n80device.cmt.read();

	case 0x40: {
				BYTE v = (n80device.vrtc.poll(n80device.io._i40) & 0xfe);
				if (n80device.fdu.available(&n80device.fdu))
					v &= 0xf7;
				else
					v |= 0x08;
				return v;
			}

	case 0xf0:
		switch (port & 0x0f) {
		case 0x0c: {// 8255(FDU) Port.A
#if _DEBUG
			BYTE v = n80device.fdu.ppi.extl.inA();
			send_fdu_command_cnt = 0;
			if (recv_fdu_command_remain == -1) {
				if ((recv_fdu_command_cnt++ % 16) == 15)
					n80device.logger("%2.2X \n", v);
				else
					n80device.logger("%2.2X ", v);
			}
			else if (recv_fdu_command_remain == 0)
				n80device.logger(" <= [%2.2X] \n", v);
			else if (++recv_fdu_command_cnt == recv_fdu_command_remain)
				n80device.logger("%2.2X \n", v);
			else
				n80device.logger(" <= [%2.2X] \n", v);
			return v;
#else
			return n80device.fdu.ppi.extl.inA();
#endif
		}
		case 0x0d:	// 8255(FDU) Port.B
			return n80device.fdu.ppi.extl.inB();
		case 0x0e:	// 8255(FDU) Port.C
			return n80device.fdu.ppi.extl.inC();
		case 0x0f:	// 8255(FDU) Mode
			return n80device.fdu.ppi.extl.inM();
		}
		break;
	}
	return 0xff;
}

static void __cdecl _n80_iowrite(LPZ80CPU cpu, USHORT port, BYTE v)
{
	BYTE portno = port & 0x0f;
	switch ((port & 0xf0)) {
	case 0x00:
			if (n80device.param.use_new_pcg) {
				BYTE regB = (port >> 8) & 0xff;
				n80device.chiqlappe.mode(regB, portno, v);
			} else
				n80device.hal.mode(portno, v);
			break;

	case 0x10:
			n80device.io._o10 = v;
			break;

	case 0x20:
			if ((portno & 0x01) == 0) {
				n80device.io._o20 = v;
				n80device.cmt.write(v);
			} else {
				n80device.io._o21 = v;
				n80device.cmt.cmd(v);
			}
			break;

	case 0x30:
			n80device.crtc.mode(v);
			break;

	case 0x40:
			n80device.io._o40 = v;
			n80device.calender.mode();
			if (v & _PO40_BEEP)
				n80device.snd.beep_on();
			else
				n80device.snd.beep_off();
			break;

	case 0x50:
			if (port & 0x01)
				n80device.crtc.cmd(v);
			else
				n80device.crtc.data(v);
			break;
			
	case 0x60:
			switch (portno) {
			case 0:	n80device.dmac.ch_adr(0, v);	break;
			case 1:	n80device.dmac.ch_cnt(0, v);	break;
			case 2:	n80device.dmac.ch_adr(1, v);	break;
			case 3:	n80device.dmac.ch_cnt(1, v);	break;
			case 4:	n80device.dmac.ch_adr(2, v);	break;
			case 5:	n80device.dmac.ch_cnt(2, v);	break;
			case 6:	n80device.dmac.ch_adr(3, v);	break;
			case 7:	n80device.dmac.ch_cnt(3, v);	break;
			case 8:	n80device.dmac.cmd(v);			break;
			}
			break;
	case 0xf0:
		switch (port & 0x0f) {
		case 0x0c:	// 8255(FDU) Port.A
			n80device.fdu.resume(&n80device.fdu);
			n80device.fdu.ppi.extl.outA(v);
			break;
		case 0x0d:	// 8255(FDU) Port.B
#if _DEBUG
			if (send_fdu_command_cnt++ == 0) {
				n80device.logger("PC: [8255] send command <{%2.2x}", v);
				send_fdu_command_code = v;
				recv_fdu_command_cnt = 0;
				recv_fdu_command_remain = 0;
				switch (v) {
				case 0x03:	recv_fdu_command_remain = -1;
				case 0x00:
				case 0x06:
				case 0x07:
				case 0x08:
				case 0x10:
				case 0x18:
				case 0x19:
				case 0x1a:
				case 0x1b:
					send_fdu_command_remain = 1;	break;
				case 0x13:	recv_fdu_command_remain = 8;
					send_fdu_command_remain = 1;	break;
				case 0x01:
				case 0x11:
				case 0x16:
					send_fdu_command_remain = -1;	break;
				case 0x09:	recv_fdu_command_remain = -1;
				case 0x0b:	recv_fdu_command_remain = -1;
				case 0x15:	recv_fdu_command_remain = -1;
				case 0x02:
				case 0x0c:
					send_fdu_command_remain = 5;	break;
				case 0x04:
					send_fdu_command_remain = 8;	break;
				case 0x12:	recv_fdu_command_remain = -1;
				case 0x05:
				case 0x0a:
				case 0x14:
				case 0x17:
				case 0x1e:
					send_fdu_command_remain = 2;	break;
				case 0x0d:
				case 0x1c:
				case 0x1f:
					send_fdu_command_remain = 3;	break;
				case 0x0e:
				case 0x0f:
					send_fdu_command_remain = 7;	break;
				case 0x1d:
					send_fdu_command_remain = 4;	break;
				}
			}
			else {
				if (send_fdu_command_cnt == 1)
					n80device.logger(" <%2.2x", v);
				else if ((send_fdu_command_cnt - 1) % 16 == 15)
					n80device.logger(", %2.2x\n", v);
				else
					n80device.logger(", %2.2x", v);
			}
			if (send_fdu_command_remain > 0 && send_fdu_command_cnt >= send_fdu_command_remain) {
				n80device.logger(">\n");
				send_fdu_command_cnt = 0;
			}
#endif
			n80device.fdu.resume(&n80device.fdu);
			n80device.fdu.ppi.extl.outB(v);
			break;
		case 0x0e:	// 8255(FDU) Port.C
			n80device.fdu.resume(&n80device.fdu);
			n80device.fdu.ppi.extl.outC(v);
			break;
		case 0x0f:	// 8255(FDU) Mode register
			n80device.fdu.resume(&n80device.fdu);
			n80device.fdu.ppi.extl.outM(v);
			break;
		}
	}
}
	
static void _n80_clear_keys(void) {
	int i;
	for (i = 0; i < 12; ++i)
		n80device.io.keys[i] = 0xff;
}

static void _n80_io_set_vector(void);
static void _n80_clear_port(void) {
	memset(&n80device.io, 0, sizeof(N80IOPORT));
	n80device.io._i40 = 0x08;	// PC-8031 N/A
	_n80_clear_keys();
	_n80_io_set_vector();
}

static void _n80_io_set_vector(void) {
	n80device.io.read		= _n80_ioread;
	n80device.io.write		= _n80_iowrite;
	n80device.io.clear		= _n80_clear_port;
	n80device.io.clear_keys	= _n80_clear_keys;
}

export void _n80_init_ioport(void) {
	_n80_clear_port();
}
#ifdef __cplusplus
}	// extern "C"
#endif
