/**==========================================================================

	PC-8001 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2010 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	External assembler functions for IBM-PC with VC++ 5.0
	last modified  Apr,4,1998.

;==========================================================================*/
#include "PC-8001.h"
#include <memory.h>
#include <math.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#ifdef _MSC_VER
#include <crtdbg.h>
#else
#include <sys/time.h>
#endif
#ifdef __cplusplus
extern "C"
{
#endif
#define export
extern void __cdecl	__z80_set_external_method(LPZ80CPU cpu, LPZ80FNCMBR mbr, LPZ80FNCMBW mbw, LPZ80FNCIOR ior, LPZ80FNCIOW iow);
extern void __cdecl _fdu_initialize(LPFDU fdu, LPN80DEVICE);	// PC-8031.c
extern void _n80_init_memory(void);								// MEMORY.c
extern void _n80_init_ioport(void);								// IO.c
extern void _n80_init_attr(void);								// ATTRIB.c
extern void _n80_init_calender(void);							// PC1990.c
extern void _n80_init_crtc(void);								// CRTC.c
extern void _n80_init_dmac(void);								// DMAC.c
extern void _n80_init_vrtc(void);								// VRTC.c
extern void _n80_init_cursor(void);								// CURSOR.c
extern void _n80_init_cmt(void);								// CMT.c
extern void _n80_init_pcg(void);								// PCG.c
extern void _n80_init_new_pcg(void);							// NEWPCG.c
extern void	_n80_init_blt(void);								// BLT.c
extern void _n80_init_snd(void);								// alsa/DirectSound
#ifndef _MSC_VER
N80DEVICE	n80device;
#endif
/**--------------------------------------------------------------------------
 **		N80 Device Methods
 **------------------------------------------------------------------------*/
#if _DEBUG
static void _n80_logger(const char* fmt, ...) {
	char	msg[1024];
	memset(msg, sizeof(msg), 1);
	va_list arg;
	va_start(arg, fmt);
	vsprintf_s(msg, sizeof(msg), fmt, arg);
	va_end(arg);
#if _MSC_VER
	_RPT0(_CRT_WARN, msg);
#else
	fprintf(strerr, msg);
#endif
}
#endif
#if defined(_MSC_VER) || defined(__APPLE__)
static void _n80_calc_clk_sync(LPN80DEVICE dev, DWORD hz, DWORD interval) {
	dev->param.hz = hz;
	dev->param.sync_interval = interval;

	dev->param.sync_ticks = (UINT) floor(((double) dev->param.hz) * dev->param.sync_interval / 1000.0 + 0.5);
}
#else
static void calc_clk_sync(LPN80DEVICE dev, DWORD hz, double interval) {
	dev->param.hz = hz;
	dev->param.interval_sec = interval;
	dev->param.sync_ticks = (UINT) floor(((double) dev->param.hz) * dev->param.interval_sec + 0.5);
}
#endif
#ifdef _MSC_VER
static double dbl_performance_freq;
static volatile double _n80_get_tick(void) {
	LARGE_INTEGER tick;
	QueryPerformanceCounter(&tick);
	return ((double)tick.QuadPart / dbl_performance_freq);
}
static void _n80_initialize(void) {
	LARGE_INTEGER freq;
	QueryPerformanceFrequency(&freq);
	dbl_performance_freq = (double)freq.QuadPart;
}
#else
static double _n80_get_tick(void) {
	struct timeval time;
	gettimeofday(&time, NULL);
	return time.tv_sec + (double)time.tv_usec * 1e-6;
}
static void _n80_initialize(void) {
}
#endif
static DWORD _n80_msec_to_T(LPN80DEVICE dev, double msec) {
	return (DWORD)floor(((double)dev->param.hz) * (msec / 1000.0) + 0.5);
}
static void _n80_device_reset(LPN80DEVICE dev) {
	dev->io.clear();
	dev->mem.clear();
	
	dev->attr.invalid();
	dev->calender.reset();
	dev->crtc.reset();
	dev->cursor.reset();
	dev->vrtc.reset();
	dev->dmac.reset();
	//dev->cmt.reset();
	dev->fdu.reset(&dev->fdu);
}
	
static void _n80_dispatch(LPN80DEVICE dev) {
	//give a chance to process to PC-8001 peripheralls
	dev->calender.clk(dev->Ttns);	// calender clk
}
static void _n80_dispose(LPN80DEVICE dev) {
	dev->fdu.dispose(&dev->fdu);
}
export void _n80_device_initialize(LPN80DEVICE dev, SYNCFNC enter, SYNCFNC leave)
{
	// clear parameters
	memset(dev, 0, sizeof(N80DEVICE));

	// set methods
	dev->dispatch			= _n80_dispatch;
	dev->reset				= _n80_device_reset;
	dev->dispose			= _n80_dispose;
	dev->io.init			= _n80_init_ioport;		// IO.c
	dev->mem.init			= _n80_init_memory;		// MEMORY.c
	dev->attr.init			= _n80_init_attr;		// ATTRIB.c
	dev->calender.init		= _n80_init_calender;	// PC1990.c
	dev->crtc.init			= _n80_init_crtc;		// CRTC.c
	dev->cursor.init		= _n80_init_cursor;		// CURSOR.c
	dev->vrtc.init			= _n80_init_vrtc;		// VRTC.c
	dev->dmac.init			= _n80_init_dmac;		// DMAC.c
	dev->cmt.init			= _n80_init_cmt;		// CMT.c
	dev->hal.init			= _n80_init_pcg;		// PCG.c
	dev->chiqlappe.init		= _n80_init_new_pcg;	// NEWPCG.c
	dev->blt.init			= _n80_init_blt;		// BLT.c
	dev->snd.init			= _n80_init_snd;		// alsa/DirectSound
	dev->vram_sync_enter	= enter;
	dev->vram_sync_leave	= leave;
	dev->calc_clk_sync		= _n80_calc_clk_sync;
	dev->msec_to_T			= _n80_msec_to_T;
	dev->get_tick			= _n80_get_tick;
#if _DEBUG
	dev->logger				= _n80_logger;
#endif
	
	// call initial methods
	_n80_initialize();
	dev->mem.init();
	dev->io.init();
	dev->attr.init();
	dev->calender.init();
	dev->crtc.init();
	dev->cursor.init();
	dev->vrtc.init();
	dev->dmac.init();
	dev->cmt.init();
	dev->hal.init();
	dev->chiqlappe.init();
	dev->blt.init();
	dev->snd.init();
	dev->attr.chg_converter(1);						// attribute convert mode init (mono)
#ifndef __linux__
    dev->calc_clk_sync(dev, 3993600, IVLMSEC);		// CLK = 4MHz, sync interval = 20ms
#else
	dev->calc_clk_sync(dev, 3993600, IVLSEC);		// CLK = 4MHz, sync interval = 10ms
#endif

	//setup PC-8031
	_fdu_initialize(&dev->fdu, dev);

	// setup main/sub cpu
	__z80_set_external_method(&dev->cpu, dev->mem.read, dev->mem.write, dev->io.read, dev->io.write);
	dev->cpu.id = 1;	// set user defined cpu id
}

#ifdef __cplusplus
}	// extern "C"
#endif
