/**==========================================================================

	PC-8001 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2010 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	External assembler functions for IBM-PC with VC++ 5.0
	last modified  Apr,4,1998.

;==========================================================================*/
#include "PC-8001.h"
#ifdef _MSC_VER
#include <windows.h>
#else
#include <time.h>
#endif
#ifdef __cplusplus
extern "C"
{
#endif
#define export
/**--------------------------------------------------------------------------
 **		ASM Modules
 **------------------------------------------------------------------------*/
static __inline void _lsb_set(void)
{
	BYTE al, ah;
	al = n80device.io._i40;
	al &= 0xef;
	ah = ((BYTE)n80device.calender._shift_reg & 0x01);
	if (ah & 0x01)
		al |= 0x10;
	n80device.io._i40 = al;
}

static __inline BYTE bcd2hex(BYTE b) {
	return ((b & 0xf0) >> 4) * 10 + (b & 0x0f);
}

static __inline BYTE hex2bcd(USHORT t)
{
	USHORT tt = t % 100;
	USHORT h = tt / 10;
	USHORT l = tt % 10;
	h <<= 4;
	h += l;
	return (BYTE) (h & 0xff);
}

static __inline void _set_sreg(void)
{	// set current date/time to uPD1990 40bits Shift Register
	ULONGLONG r = n80device.calender._time_counter;
	n80device.calender._sec = bcd2hex((BYTE)(r & 0xff));	r >>= 8;
	n80device.calender._min = bcd2hex((BYTE)(r & 0xff));	r >>= 8;
	n80device.calender._hour = bcd2hex((BYTE)(r & 0xff));	r >>= 8;
	n80device.calender._day = bcd2hex((BYTE)(r & 0xff));	r >>= 12;
	n80device.calender._month = (BYTE)(r & 0x0f);

	n80device.calender._sec++;
	if (n80device.calender._sec > 59) {
		n80device.calender._sec -= 60;
		n80device.calender._min += 1;
	}
	if (n80device.calender._min > 59) {
		n80device.calender._min -= 60;
		n80device.calender._hour += 1;
	}
	if (n80device.calender._hour > 23) {
		n80device.calender._hour -= 24;
		n80device.calender._day += 1;
	}
	if (n80device.calender._day > 31) {
		n80device.calender._day -= 31;
		n80device.calender._month += 1;
	}
	if (n80device.calender._month > 12) {
		n80device.calender._month -= 12;
	}
	r = 0;
	r += (BYTE)(n80device.calender._month);
	r <<= 12;
	r += hex2bcd((BYTE)(n80device.calender._day));
	r <<= 8;
	r += hex2bcd((BYTE)(n80device.calender._hour));
	r <<= 8;
	r += hex2bcd((BYTE)(n80device.calender._min));
	r <<= 8;
	r += hex2bcd((BYTE)(n80device.calender._sec));
	n80device.calender._time_counter = r;
}

export void __cdecl _n80_time_mode(void)
{
	BYTE ah, al, bl;
	ah = n80device.io._o40;
	// calender strobe signal check
	bl = ah & 0x02;
	al = n80device.calender._cstrb & 0x02;
	al ^= bl;
	// strobe edge detect
	if (al != 0) {
		n80device.calender._cstrb = bl;
		// strobe edge type 1 -> 0
		if (bl == 0) {
			// uPC1990 command
			bl = n80device.io._o10 & 0x07;
			switch (bl) {
			case 0x00:	// cmd 0: Register Hold Command
						n80device.calender._shiften = 0; break;
			case 0x01:	// cmd 1: Register Shift Command
						n80device.calender._shiften = 1; break;
			case 0x02:	// cmd 2: Time Set Command
						n80device.calender._shiften = 0;
						n80device.calender._time_counter = n80device.calender._shift_reg;
						_lsb_set(); break;
			case 0x03:	// cmd 3: Time Read Command
						n80device.calender._shiften = 0;
						n80device.calender._shift_reg = n80device.calender._time_counter;
						_lsb_set(); break;
			case 0x04:	// cmd 4: TP=64Hz Command
			case 0x05:	// cmd 5: TP=256Hz Command
			case 0x06:	// cmd 6: TP=2048Hz Command
			case 0x07:	// cmd 7: Test Command
			default:	break;
			}
		}
	}
	// CLK strobe check
	ah = n80device.io._o40 & 0x04;
	if (n80device.calender._shiften == 0) {
		n80device.calender._cclk = ah;
		return;
	}
	al = n80device.calender._cclk & 0x04;
	al ^= ah;
	if (al == 0)
		return;
	// detect CLK strobe
	n80device.calender._cclk = ah;
	// CLK strobe edge type 1 -> 0
	if (ah == 0) {
		// 40bit register shift 
		n80device.calender._shift_reg >>= 1;
		if (n80device.io._o10 & 0x08)
			n80device.calender._shift_reg |= 0x8000000000;
		else
			n80device.calender._shift_reg &= 0x7fffffffff;
		_lsb_set();
	}
}
static void _n80_clk_calender(int tick) {
	n80device.calender._clk_counter += tick;
	if (n80device.calender._clk_counter >= 100000000) {
		n80device.calender._clk_counter -= 100000000;
		_set_sreg();
	}
}

static void _n80_reset_calender(void) {
	n80device.calender._cstrb = 0;
	n80device.calender._cclk = 0;
	n80device.calender._shiften = 0;
	n80device.calender._shift_reg = 0;
	n80device.calender._time_counter = n80device.calender._shift_reg = 0;
	n80device.calender._clk_counter = 0;
	n80device.io._i40 = 0x0d;
	_set_sreg();
}

export void __cdecl _n80_init_calender(void) {
	n80device.calender.mode		= _n80_time_mode;
	n80device.calender.reset	= _n80_reset_calender;
	n80device.calender.clk		= _n80_clk_calender;
	_n80_reset_calender();
}
	
#ifdef __cplusplus
}	// extern "C"
#endif
