/**==========================================================================

	PC-8031 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2024 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	last modified  Apr,27,2024.

;==========================================================================*/
#include "../PC-8001/PC-8001.h"
#ifdef __cplusplus
extern "C"
{
#endif
#define export
/**--------------------------------------------------------------------------
 **		ASM Modules (port hi=B reg, lo=port no)
 **------------------------------------------------------------------------*/
static BYTE __cdecl _fdu_ioread(LPFDU fdu, USHORT port)
{
	LPFDU765A fdc = &fdu->fdc;
	USHORT portno = port & 0xff;
	switch (portno) {
	case 0xf4:
		return 0x00;
	case 0xf8:
		fdc->TC(fdc);
		return 0xff;
	case 0xfa:		// FDC status
		fdu->resume(fdu);
		return fdc->read_status_reg(fdc);
	case 0xfb:		// FDC data
		fdu->resume(fdu);
		return fdc->read(fdc);
	case 0xfc:		// 8255(FDU) Port.A
#if _DEBUG
		//n80device.logger("FDU: => [8255] recv command {%2.2X}\n", fdu->ppi.intl.inA());
#endif
		return fdu->ppi.intl.inA();
	case 0xfd:		// 8255(FDU) Port.B
		return fdu->ppi.intl.inB();
	case 0xfe:		// 8255(FDU) Port.C
		return fdu->ppi.intl.inC();
	case 0xff:		// 8255(FDU) Mode
		return fdu->ppi.intl.inM();
	default:
		return 0xff;
	}
}

static void __cdecl _fdu_iowrite(LPFDU fdu, USHORT port, BYTE v)
{
	LPFDU765A fdc = &fdu->fdc;
	USHORT portno = port & 0xff;
	switch (portno) {
	case 0xf4:		// drive mode
		fdu->fdd.set_mode(&fdu->fdd, v);
		break;
	case 0xf7:		// pre-compensation
		break;
	case 0xf8:		// drive motor
		fdu->fdd.motor_control(&fdu->fdd, v);
		break;
	case 0xfb:		// FDC data
		fdu->resume(fdu);
		fdc->write(fdc, v);
		break;
	case 0xfc:		// 8255(FDU) Port.A
		fdu->ppi.intl.outA(v);
		break;
	case 0xfd:		// 8255(FDU) Port.B
		fdu->ppi.intl.outB(v);
#if _DEBUG
		//n80device.logger(" <= [%2.2X] \n", v);
#endif
		break;
	case 0xfe:		// 8255(FDU) Port.C
		fdu->ppi.intl.outC(v);
		break;
	case 0xff:		// 8255(FDU) Mode register
		fdu->ppi.intl.outM(v);
		break;
	default:
		break;
	}
}

static void _fdu_clear_port(LPFDU fdu) {
	fdu->io.cache._pio_A_fdu = 0;
	fdu->io.cache._pio_B_fdu = 0;
	fdu->io.cache._pio_C_fdu = 0;
	fdu->io.cache._pio_M_fdu = 0;
	fdu->io.cache._pio_A_pc = 0;
	fdu->io.cache._pio_B_pc = 0;
	fdu->io.cache._pio_C_pc = 0;
	fdu->io.cache._pio_M_pc = 0;
}

export void _fdu_init_ioport(void) {
	n80device.fdu.io.read	= _fdu_ioread;
	n80device.fdu.io.write	= _fdu_iowrite;
	n80device.fdu.io.clear	= _fdu_clear_port;
	n80device.fdu.io.clear(&n80device.fdu);
}
#ifdef __cplusplus
}	// extern "C"
#endif
