/**==========================================================================

	PC-8031 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2024 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	last modified  Apr,27,2024.

;==========================================================================*/
#include "../PC-8001/PC-8001.h"
#ifdef __cplusplus
extern "C"
{
#endif
#define export
static const char* err_size = "Described size exceeds actual image length\n";
static const char* err_head = "Header size is shorter than specified\n";
static const char* err_trkz = "Track data exceeds image length\n";
static const char* err_sect = "Sector data exceeds image length\n";
static const char* err_scnt = "Amount of Sectors is over 32\n";
static const char* err_id_c = "wrong cylinder id\n";
static const char* err_id_h = "wrong head id\n";
static const char* err_id_r = "wrong sector id\n";
/**--------------------------------------------------------------------------
 **		METHODS
 **------------------------------------------------------------------------*/
static USHORT read_short(LPBYTE ptr) {
	USHORT v = 0;
	v = (USHORT)ptr[1];
	v <<= 8;
	v += (USHORT)ptr[0];
	return v;
}
static DWORD read_quad(LPBYTE ptr) {
	DWORD v = 0;
	v = (DWORD)ptr[3];
	v <<= 8;
	v += (DWORD)ptr[2];
	v <<= 8;
	v += (DWORD)ptr[1];
	v <<= 8;
	v += (DWORD)ptr[0];
	return v;
}

static int _d88_sector_count(LPD88SECTOR self) {
	return (int)read_short(self->sectors);
}

static int _d88_sector_length(LPD88SECTOR self) {
	return (int)read_short(self->length);
}

static BOOL _media_check_image(LPFLOPPYDISK self) {
	self->header = (LPD88HEADER)self->image_raw_data;
	// LڂĂ黲ނۂ̲Ұޒ𒴉߂Ăꍇʹװ
	if (read_quad(self->header->disksize) > self->image_length) {
		self->err_msg = err_size;
		return FALSE;
	}
	// ͯް̈悪Ұޒ𒴉߂Ăꍇʹװ
	if (self->image_length <= sizeof(D88HEADER)) {
		self->err_msg = err_head;
		return FALSE;
	}
	memset(&self->offsets, 0xff, sizeof(self->offsets));
	// ׯ
	for (int t = 0, t2 = 0; t < 164; ++t) {
		BYTE C = (BYTE)(t / 2);	// cylinder
		BYTE H = (BYTE)(t % 2);	// head

		// {ׯިł1ްƂɽ߂
		if (self->header->disktype == 0x00)
			t2 = t;		// 2D
		else if ((t % 4) > 2)
			continue;	// 2DD/2HD̼ް1΂ɂ
		else {			// ΂l߂
			C = t / 4;
			t2 = ((t / 4) * 2) + (t % 2);
		}
		DWORD offset = read_quad(&self->header->trackptr[t * 4]);
		if (offset == 0)
			continue;
		// ׯ̾ĂD88ͯް̈ɂꍇʹװ
		if (offset < 32)
			continue;
		// ͯް̈悪Ұޒ𒴉߂Ăꍇʹװ
		if ((offset + 16) > self->image_length) {
			self->err_msg = err_trkz;
#if _DEBUG
			n80device.logger(self->err_msg);
#endif
			continue;
		}
		LPD88SECTOR track = (LPD88SECTOR)&self->image_raw_data[offset];
		USHORT sectors = read_short(track->sectors);
		if (sectors > 32) {
			self->err_msg = err_scnt;
#if _DEBUG
			n80device.logger(self->err_msg);
#endif
			continue;
		}
#if _DEBUG
		n80device.logger("track offsets[%d](C:%d/H:%d) : ", t, track->id.C, track->id.H);
#endif
		for (int s = 0; s < sectors; ++s) {
			BYTE R = (BYTE)(s + 1);
			USHORT sectlen = read_short(track->length);
			// ް̈悪Ұޒ𒴉߂Ăꍇʹװ
			if ((offset + sectlen + 16) > self->image_length) {
				self->err_msg = err_sect;
#if _DEBUG
				n80device.logger(self->err_msg);
#endif
				break;
			}
			if (track->id.C != C) {
				self->err_msg = err_id_c;
#if _DEBUG
				//n80device.logger(self->err_msg);
#endif
				//return FALSE;
			}
			if (track->id.H != H) {
				self->err_msg = err_id_h;
#if _DEBUG
				//n80device.logger(self->err_msg);
#endif
				//return FALSE;
			}
			if (track->id.R != R) {
				self->err_msg = err_id_r;
#if _DEBUG
				//n80device.logger(self->err_msg);
#endif
				//return FALSE;
			}
			self->offsets[t2][s] = offset;
#if _DEBUG
			n80device.logger("[%d:{%d,%d,%d,%d}],", s, track->id.C, track->id.H, track->id.R, track->id.N);
#endif
			offset += sectlen + 16;
			track = (LPD88SECTOR)&self->image_raw_data[offset];
		}
#if _DEBUG
		n80device.logger("\n");
#endif
	}
	return TRUE;
}

static int _d88_find_track_index(LPFLOPPYDISK self, int cylinder, BYTE HD) {
	int t = cylinder * 2 + HD;
	if (t >= 0 && t < 164)
		return t;
	return -1;
}

static LPD88SECTOR_EX _d88_enum_track_sector(LPFLOPPYDISK self, int t, int s) {
	LPD88SECTOR_EX sector = NULL;
	if (t >= 0 && t < 164 && s >= 0 && s <= 32)
		sector = self->get_sector(self, t * 100 + s);
	return sector;
}

static LPD88SECTOR_EX _d88_get_sector(LPFLOPPYDISK self, int i) {
	if (i >= 0) {
		int t = i / 100;
		int s = i % 100;
		if (t >= 0 && t < 164 && s >= 0 && s < 32) {
			DWORD offset = self->offsets[t][s];
			if (offset != 0xffffffff)
				return (LPD88SECTOR_EX)&self->image_raw_data[offset];
		}
	}
	return NULL;
}

static LPD88SECTOR_EX _d88_find_sector_in_track(LPFLOPPYDISK self, int cylinder, BYTE HD, BYTE MT, LPSECTORID id, int *track, int *sectno, int *header) {
	int t = cylinder * 2 + HD;
	for (int h = 0; h < 2; ++h) {
		if (t >= 0 && t < 164) {
			*track = t;
			*header = HD;
			for (int s = 0; s < 32; ++s) {
				DWORD offset = self->offsets[t][s];
				if (offset != 0xffffffff) {
					LPD88SECTOR sector = (LPD88SECTOR)&self->image_raw_data[offset];
					*sectno = s;
					if (sector->id.C == id->C && sector->id.H == id->H && sector->id.R == id->R) {
						return (LPD88SECTOR_EX)sector;
					}
				} else {
					break;
				}
			}
			if (HD == 0 && MT == 1) {
				++t;
				HD = 1;
				*sectno = -1;
			} else
				break;
		}
	}
	return NULL;
}

static void _d88_release_floppy(LPFLOPPYDISK self) {
	if (self->mounted) {
		self->mounted = FALSE;
		self->image->dispose(self->image);
	}
}
/**--------------------------------------------------------------------------
 **		Floppy Disk Initializer
 **------------------------------------------------------------------------*/
static BOOL _n88_init_media(LPFLOPPYDISK self, LPFDU fdu) {
	self->find_track_index		= _d88_find_track_index;
	self->enum_track_sector		= _d88_enum_track_sector;
	self->get_sector			= _d88_get_sector;
	self->find_sector_in_track	= _d88_find_sector_in_track;
	self->sectors				= _d88_sector_count;
	self->length				= _d88_sector_length;
	return _media_check_image(self);
}
/**--------------------------------------------------------------------------
 **		Disk Image Initial Scan
 **------------------------------------------------------------------------*/
export BOOL _fdu_image_scan(LPFDU fdu, LPDISKIMAGE img) {
	for (DWORD base_offset = 0, bottom_offset = 0; base_offset < img->image_length; base_offset = bottom_offset) {
		LPD88HEADER header = (LPD88HEADER)&img->image_raw_data[base_offset];
		int length = read_quad(header->disksize);
		bottom_offset = base_offset + length;
		if (img->disk_count < 10) {
			LPFLOPPYDISK disk = img->alloc();
			if (disk == NULL)
				break;
			img->disks[img->disk_count++] = disk;
			disk->image				= img;
			disk->base_offset		= base_offset;
			disk->bottom_offset		= bottom_offset;
			disk->image_length		= length;
			disk->image_raw_data	= (LPBYTE)header;
			disk->header			= header;
			disk->dispose			= _d88_release_floppy;
			disk->mounted			= FALSE;
			if (!_n88_init_media(disk, fdu))
				break;
		}
	}
	return img->disk_count > 0;
}

#ifdef __cplusplus
}	// extern "C"
#endif
