/**==========================================================================

	PC-8031 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2024 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	last modified  Apr,27,2024.

;==========================================================================*/
#include "../PC-8001/PC-8001.h"
#ifdef __cplusplus
extern "C"
{
#endif
#define export
#define PPI_C_PC_TO_FDU	(0)
#define PPI_C_FDU_TO_PC	(1)
/**--------------------------------------------------------------------------
 **		METHODS
 **------------------------------------------------------------------------*/
static void _pio_port_c_write(BYTE dir, BYTE data) {
	if (dir == PPI_C_PC_TO_FDU)
		n80device.fdu.io.cache._pio_C_pc = data;
	else
		n80device.fdu.io.cache._pio_C_fdu = data;
}
static void _pio_port_c_bit_control(BYTE dir, BYTE data) {
	BYTE reg = dir == PPI_C_PC_TO_FDU ? n80device.fdu.io.cache._pio_C_pc : n80device.fdu.io.cache._pio_C_fdu;
	BYTE bit = 0x01;
	int shift = 0;
	shift = ((data & 0x0e) >> 1) & 0x07;
	bit <<= shift;
	if ((data & 0x01) == 1) {
		reg |= bit;
	} else
		reg &= ~bit;
	_pio_port_c_write(dir, reg);
#if _DEBUG
	//switch (shift) {
	//case 4:
	//	n80device.logger("%s: [8255] DAV => \"%d\"\n", dir == PPI_C_PC_TO_FDU ? "PC " : "FDU", data & 0x01);	break;
	//case 5:
	//	n80device.logger("%s: [8255] RFD => \"%d\"\n", dir == PPI_C_PC_TO_FDU ? "PC " : "FDU", data & 0x01);	break;
	//case 6:
	//	n80device.logger("%s: [8255] DAC => \"%d\"\n", dir == PPI_C_PC_TO_FDU ? "PC " : "FDU", data & 0x01);	break;
	//case 7:
	//	n80device.logger("%s: [8255] ATN => \"%d\"\n", dir == PPI_C_PC_TO_FDU ? "PC " : "FDU", data & 0x01);	break;
	//}
#endif
}
/**--------------------------------------------------------------------------
 **		PD8255 in PC-8801 side
 **------------------------------------------------------------------------*/
static void _pio_pc_out_a(BYTE data) {
	n80device.fdu.io.cache._pio_A_pc = data;
	n80device.fdu.resume(&n80device.fdu);
}
static void _pio_pc_out_b(BYTE data) {
	n80device.fdu.io.cache._pio_B_pc = data;
	n80device.fdu.ppi.state1 = 'W';
	n80device.fdu.resume(&n80device.fdu);
#if _DEBUG
	//n80device.logger("FDU: [8255-1] PC=>FDU DATA = 0x%2.2X(%d)\n", data, data);
#endif
}
static void _pio_pc_out_c(BYTE data) {
	_pio_port_c_write(PPI_C_PC_TO_FDU, data);
	n80device.fdu.resume(&n80device.fdu);
}
static void _pio_pc_out_mode(BYTE data) {
	n80device.fdu.resume(&n80device.fdu);
	if ((data & 0x80) == 0) {	// bit manipulation
		_pio_port_c_bit_control(PPI_C_PC_TO_FDU, data);
	} else {					// mode control
		n80device.fdu.io.cache._pio_M_pc = data;
		n80device.fdu.io.cache._pio_A_pc = 0;
		n80device.fdu.io.cache._pio_B_pc = 0;
		n80device.fdu.io.cache._pio_C_pc = 0;
	}
}
static BYTE _pio_pc_in_a(void) {
	n80device.fdu.ppi.state1 = 'R';
	n80device.fdu.resume(&n80device.fdu);
	return n80device.fdu.io.cache._pio_B_fdu;
}
static BYTE _pio_pc_in_b(void) {
	n80device.fdu.resume(&n80device.fdu);
	return n80device.fdu.io.cache._pio_A_fdu;
}
static BYTE _pio_pc_in_c(void) {
	n80device.fdu.resume(&n80device.fdu);
	return (n80device.fdu.io.cache._pio_C_pc & 0xf0) | ((n80device.fdu.io.cache._pio_C_fdu >> 4) & 0x0f);
}
static BYTE _pio_pc_in_mode(void) {
	n80device.fdu.resume(&n80device.fdu);
	return n80device.fdu.io.cache._pio_M_pc;
}
/**--------------------------------------------------------------------------
 **		PD8255 in PC-8031 side
 **------------------------------------------------------------------------*/
static void _pio_fdu_out_a(BYTE data) {
	n80device.fdu.io.cache._pio_A_fdu = data;
}
static void _pio_fdu_out_b(BYTE data) {
	n80device.fdu.io.cache._pio_B_fdu = data;
	n80device.fdu.ppi.state2 = 'W';
#if _DEBUG
	//n80device.logger("FDU: [8255-2] FDU=>PC DATA= 0x%2.2X(%d)\n", data, data);
#endif
}
static void _pio_fdu_out_c(BYTE data) {
	_pio_port_c_write(PPI_C_FDU_TO_PC, data);
}
static void _pio_fdu_out_mode(BYTE data) {
	if ((data & 0x80) == 0) {	// bit manipulation
		_pio_port_c_bit_control(PPI_C_FDU_TO_PC, data);
	} else {					// mode control
		n80device.fdu.io.cache._pio_M_fdu = data;
		n80device.fdu.io.cache._pio_A_fdu = 0;
		n80device.fdu.io.cache._pio_B_fdu = 0;
		n80device.fdu.io.cache._pio_C_fdu = 0;
	}
}
static BYTE _pio_fdu_in_a(void) {
	n80device.fdu.ppi.state2 = 'R';
	return n80device.fdu.io.cache._pio_B_pc;
}
static BYTE _pio_fdu_in_b(void) {
	return n80device.fdu.io.cache._pio_A_pc;
}
static BYTE _pio_fdu_in_c(void) {
	return (n80device.fdu.io.cache._pio_C_fdu & 0xf0) | ((n80device.fdu.io.cache._pio_C_pc >> 4) & 0x0f);
}
static BYTE _pio_fdu_in_mode(void) {
	return n80device.fdu.io.cache._pio_M_fdu;
}

export void _fdu_init_fduif(void) {
	n80device.fdu.ppi.extl.outA  = _pio_pc_out_a;
	n80device.fdu.ppi.extl.outB  = _pio_pc_out_b;
	n80device.fdu.ppi.extl.outC  = _pio_pc_out_c;
	n80device.fdu.ppi.extl.outM  = _pio_pc_out_mode;
	n80device.fdu.ppi.extl.inA   = _pio_pc_in_a;
	n80device.fdu.ppi.extl.inB   = _pio_pc_in_b;
	n80device.fdu.ppi.extl.inC   = _pio_pc_in_c;
	n80device.fdu.ppi.extl.inM   = _pio_pc_in_mode;

	n80device.fdu.ppi.intl.outA = _pio_fdu_out_a;
	n80device.fdu.ppi.intl.outB = _pio_fdu_out_b;
	n80device.fdu.ppi.intl.outC = _pio_fdu_out_c;
	n80device.fdu.ppi.intl.outM = _pio_fdu_out_mode;
	n80device.fdu.ppi.intl.inA  = _pio_fdu_in_a;
	n80device.fdu.ppi.intl.inB  = _pio_fdu_in_b;
	n80device.fdu.ppi.intl.inC  = _pio_fdu_in_c;
	n80device.fdu.ppi.intl.inM  = _pio_fdu_in_mode;

	n80device.fdu.ppi.state1	= '*';
	n80device.fdu.ppi.state2	= '*';
}
#ifdef __cplusplus
}	// extern "C"
#endif
