/**==========================================================================

	PC-8031 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2024 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	last modified  Apr,27,2024.

;==========================================================================*/
#include "../PC-8001/PC-8001.h"
#ifdef __cplusplus
extern "C"
{
#endif
#define export
/**--------------------------------------------------------------------------
 **		METHODS
 **------------------------------------------------------------------------*/
static void _fdc_result_let_id(LPFDURESULT self, LPSECTORID id) {
	self->id = *id;
}
static void _fdc_result_let_std(LPFDURESULT self, LPFDUCOMMAND cmd) {
	self->ST0 = cmd->commands[1] & 0x07;	// HD/US0/US1
	self->ST1 = 0;
	self->ST2 = 0;
	self->let_id(self, &cmd->id);
}
static void _fdc_result_clear(LPFDURESULT self) {
	self->count = 0;
	self->index = 0;
	self->ptr = self->results;
}
static int _fdc_result_push0(LPFDURESULT self, BYTE data) {
	self->clear(self);
	return self->push(self, data);
}
static int _fdc_result_push(LPFDURESULT self, BYTE data) {
	self->results[self->count++] = data;
	return self->count;
}
static int _fdc_result_push_std(LPFDURESULT self) {
	self->push0(self, self->ST0);
	self->push(self, self->ST1);
	self->push(self, self->ST2);
	self->push(self, self->id.C);
	self->push(self, self->id.H);
	self->push(self, self->id.R);
	return self->push(self, self->id.N);
}
static BYTE _fdc_result_pop(LPFDURESULT self) {
	if (self->count > 0) {
		self->count--;
		return self->ptr[self->index++];
	} else
		return 0xff;
}
static void _fdc_result_copy(LPFDURESULT src, LPFDURESULT dest) {
	dest->ST0	= src->ST0;
	dest->ST1	= src->ST1;
	dest->ST2	= src->ST2;
	dest->id	= src->id;
	//dest->index = src->index;
	//dest->count = src->count;
	for (int i = 0; i < 16; ++i)
		dest->results[i] = src->results[i];
	dest->ptr = dest->results;
}
static void _fdc_result_save(LPFDURESULT self, LPFDURESULT dest) {
	dest->clear(dest);
	_fdc_result_copy(self, dest);
}
static void _fdc_result_restore(LPFDURESULT self, LPFDURESULT dest) {
	_fdc_result_copy(dest, self);
}
export void _fdc_init_result(LPFDURESULT self) {
	self->init		= _fdc_init_result;
	self->clear		= _fdc_result_clear;
	self->push0		= _fdc_result_push0;
	self->push		= _fdc_result_push;
	self->push_std	= _fdc_result_push_std;
	self->pop		= _fdc_result_pop;
	self->let_std	= _fdc_result_let_std;
	self->let_id	= _fdc_result_let_id;
	self->save		= _fdc_result_save;
	self->restore	= _fdc_result_restore;
	self->clear(self);
}
#ifdef __cplusplus
}	// extern "C"
#endif
