/**--------------------------------------------------------------------------
 **
 **		Z80 Emulator Ver 3.00
 **		Copyright (C)1993-2010 H.Kanemoto
 **
 **		This is a free software. You may copy or distribute
 **		it without any permissions. But there are NO WARRANTY
 **		of use. Entire risk of use is with you.
 **
 **------------------------------------------------------------------------*/
#include "z80.h"
#include "z80mac.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <memory.h>
#ifdef _MSC_VER
#include <crtdbg.h>
#include <mmsystem.h>
#include <windows.h>
#endif
#pragma warning(disable:4996)
#ifdef __cplusplus
extern "C"
{
#endif
#define export
/**--------------------------------------------------------------------------
 **		Z80 Decoder
 **------------------------------------------------------------------------*/
extern void _dz80_dis_decoder(Z80CPU* cpu);
/**--------------------------------------------------------------------------
 **		Z80 Debugger
 **------------------------------------------------------------------------*/
static int __cdecl __dz80_step(LPZ80CPU cpu) {
	cpu->r.x.T = 0;					// Xe[gNA
	cpu->r.x.W = 0;					// EFCgNA
	cpu->r.x.Wh = 0;				// EFCgNA(2byte)
	cpu->r.x.TwM1 = 0;				// M1 Wait
	cpu->r.x.head = cpu->r.x.PC;	// ݂̃CXgNV|C^ޔ
	cpu->fetch_cnt = 0;				// fetch code cache count
	++cpu->clk;						// R registerpNbN
	cpu->dbgctx.rbf	= cpu->r;		// sOWX^ޔ

	if (cpu->signal & _INTMASK) {
		// INT MODE 0: fetch data directry from data bus and PC not changed.
		// INT MODE 1, 2: set PC only
		if (cpu->r.x.IM != 0) {
			__z80_pushw(cpu, cpu->r.x.PC);
			cpu->r.x.PC = cpu->r.x.IPC;
			cpu->r.x.head = cpu->r.x.IPC;
		}
		cpu->signal &= ~_HLTBIT;	// HALT
		cpu->r.x.HALT = 0;
	}
	int bp = cpu->dbgctx.pre_decode(cpu);	// pre-decode and detect break pointer
	cpu->dbgctx.on_before_exec(&cpu->dbgctx);
	cpu->decoder(cpu);				// Pߎs
	if (bp != 0)
		cpu->dbgctx.on_restore_breakpt(&cpu->dbgctx, bp);
	if (cpu->signal & _INTMASK) {
		cpu->signal &= ~_INTMASK;	// INT/NMI
		cpu->r.x.T += cpu->r.x.M;	// 荞݃}VTCNZ
		cpu->r.x.M = 0;
	}
#ifdef _MSC_VER
	InterlockedExchangeAdd(&cpu->Ts, cpu->r.x.T);
#endif
#ifdef __linux__
	interlocked_exchangeadd(&cpu->Ts, cpu->r.x.T);
#endif
	cpu->dbgctx.raf = cpu->r;		// sヌWX^ޔ
	cpu->dbgctx.on_after_exec(&cpu->dbgctx);

	if (cpu->signal & _HLTBIT) {// HALT//ߌm/INT Data Bus
		return 1;
	} else
		return 0;
}

static int __cdecl __dz80_int_signal(LPZ80CPU cpu, int vector) {
	USHORT vec;
	if (cpu->r.x.IFF1 == 1) {
		cpu->r.x.IFF1 = 0;
		cpu->r.x.IFF2 = 0;
		cpu->signal |= _INTBIT;

		switch (cpu->r.x.IM) {
		case 0:	// MODE 0
			cpu->req_mode0 = 1;		// request fetch from interrup data bus
			cpu->int_dbus[0] = (BYTE)vector;
			cpu->int_dbus_cnt = 1;	// data bus buffer size
			cpu->r.x.M = 2;
			cpu->dbgctx.on_req_signal(&cpu->dbgctx);
			break;

		case 1: // MODE 1
			cpu->r.x.IPC = 0x0038;
			cpu->r.x.M = 13;
			cpu->dbgctx.on_req_signal(&cpu->dbgctx);
			break;

		case 2:	// MODE 2
			vec = (USHORT)cpu->r.x.I;
			vec <<= 8;
			vec += (vector * 2);
			cpu->r.x.M = 19;
			cpu->r.x.IPC = __z80_mwread(cpu, vec);
			cpu->dbgctx.on_req_signal(&cpu->dbgctx);
			break;

		default:	// unexpected interrupt mode
			cpu->r.x.HALT = _Z80HALT_INT;
			cpu->signal |= _HLTBIT;
			cpu->halt_detect = 1;
			return 1;
		}
		return 0;	// ACCEPTED
	}
	return 1;		// IGNORED
}

static void __cdecl __dz80_nmi_signal(LPZ80CPU cpu) {
	cpu->r.x.IFF2 = cpu->r.x.IFF1;
	cpu->r.x.IFF1 = 0;
	cpu->r.x.M = 11;		// M1(5T)+M2(3T)+M3(3T)
	cpu->signal |= _NMIBIT;
	cpu->r.x.PC = 0x0066;
	cpu->dbgctx.on_req_signal(&cpu->dbgctx);
}

static BYTE __dz80_fetchb(LPZ80CPU cpu) {
	DWORD W = 0;
	BYTE b = cpu->mbread(cpu, cpu->dbgctx.PC++, &W);
	cpu->dbgctx.bin[cpu->dbgctx.bin_w_count++] = b;
	return b;
}

static BYTE __dz80_fetchb_M1(LPZ80CPU cpu) {
	if (cpu->signal & _HLTBIT) {
		return 0x00;	// force NOP instruction duaring HALT status
	} else if (cpu->req_mode0 == 1) {
		// MODE 0: fetch from data bus which is set by interrupt device
		if (cpu->int_dbus_cnt > 0) {
			return cpu->int_dbus[0];
		} else {
			return 0x00;	//NOP
		}
	} else
		return __dz80_fetchb(cpu);
}

static int __dz80_pre_decoder(LPZ80CPU cpu) {
	cpu->dbgctx.PC = cpu->r.x.PC;
	cpu->dbgctx.mnemonic = "???";
	cpu->dbgctx.bin_w_count = 0;	// cache write counter
	cpu->dbgctx.bin_r_count = 0;	// cache read  counter
	cpu->dbgctx.opv_count = 0;		// operand value counter
	cpu->dbgctx.optype = DZ80_OPTYPE_UNKNOWN;

	_dz80_dis_decoder(cpu);
	cpu->dbgctx.PC = cpu->r.x.PC;
	if (cpu->dbgctx.bin[0] == 0x76) {
		// confirm HALT break point or not
		int bp = cpu->dbgctx.on_query_breakpt(&cpu->dbgctx);
		if (bp == 0)
			return 0;
		// replace HALT instruction to correct one
		cpu->dbgctx.on_replace_breakpt(&cpu->dbgctx, bp);
		// re-decode with correct instruction
		cpu->dbgctx.PC = cpu->r.x.PC;
		cpu->dbgctx.bin_w_count = 0;
		cpu->dbgctx.mnemonic = "???";
		_dz80_dis_decoder(cpu);
		return bp;
	} else
		return 0;
}

static void __dz80_debug_enable(LPDZ80CONTEXT dbgctx, BYTE flag) {
	BYTE enable = flag == 0 ? 0 : 1;
	if (dbgctx->enable == enable)
		return;
	if (enable == 0) {	// debug off
		dbgctx->enable		= 0;
		dbgctx->step		= dbgctx->z80_step;
		dbgctx->int_signal	= dbgctx->z80_int_signal;
		dbgctx->nmi_signal	= dbgctx->z80_nmi_signal;
	} else {			// debug on
		dbgctx->enable		= 1;
		dbgctx->step		= __dz80_step;
		dbgctx->int_signal	= __dz80_int_signal;
		dbgctx->nmi_signal	= __dz80_nmi_signal;
	}
}
static void __dz80_disasm(USHORT PC, int steps, DISCALLBACK callback) {
	//DZ80CONTEXT dbgctx;
	//
	//dbgctx.PC = PC;
	//for (int i = 0; i < steps; ++i) {
	//	dbgctx.mnemonic = "???";
	//	dbgctx.bin_w_count = 0;
	//	dbgctx.opv_count = 0;

	//}
}
static void __dz80_format_op_imm16(LPDZ80CONTEXT ctx, const char* literal, USHORT v) {
	char nn[16];
	sprintf(nn, "%4.4Xh", v);
	char* tar = strstr(ctx->mnemonic, literal);
	int len = (int)(tar - ctx->mnemonic);
	strncpy(ctx->instruction, ctx->mnemonic, len);
	ctx->instruction[len] = '\0';
	strcat(ctx->instruction, nn);
	strcat(ctx->instruction, tar + 2);
}
static void __dz80_format_op_imm8(LPDZ80CONTEXT ctx, const char *literal, BYTE v) {
	char n[8];
	sprintf(n, "%2.2Xh", v);
	char* tar = strstr(ctx->mnemonic, literal);
	int len = (int)(tar - ctx->mnemonic);
	strncpy(ctx->instruction, ctx->mnemonic, len);
	ctx->instruction[len] = '\0';
	strcat(ctx->instruction, n);
	strcat(ctx->instruction, tar + 1);
}
static const char* __dz80_format_op(LPDZ80CONTEXT ctx) {
	switch (ctx->optype) {
		case DZ80_OPTYPE_IMM16:
		case DZ80_OPTYPE_MEM16W:
		case DZ80_OPTYPE_MEM8W:
		case DZ80_OPTYPE_MEM16R:
		case DZ80_OPTYPE_MEM8R: {
			__dz80_format_op_imm16(ctx, "nn", ctx->opv[0].w);
			break;
		}
		case DZ80_OPTYPE_IMM8: {
			__dz80_format_op_imm8(ctx, "n", ctx->opv[0].b);
			break;
		}
		case DZ80_OPTYPE_OFFSET: {
			__dz80_format_op_imm16(ctx, "e", ctx->opv[1].w);
			break;
		}
		case DZ80_OPTYPE_IXD: {
			__dz80_format_op_imm8(ctx, "d", ctx->opv[0].b);
			break;
		}
		case DZ80_OPTYPE_IXD_IMM8: {
			char n[8];
			sprintf(n, "%2.2Xh", ctx->opv[0].b);
			char* tar = strstr(ctx->mnemonic, "d");
			int len = (int)(tar - ctx->mnemonic);
			strncpy(ctx->instruction, ctx->mnemonic, len);
			ctx->instruction[len] = '\0';
			strcat(ctx->instruction, n);
			char* tar2 = strstr(ctx->mnemonic, "n");
			int len2 = (int)(tar2 - tar) - 1;
			strncpy(ctx->instruction, tar + 1, len2);
			ctx->instruction[len + 3 + len2] = '\0';
			sprintf(n, "%2.2Xh", ctx->opv[1].b);
			strcat(ctx->instruction, n);
			break;
		}
		default:
			strcpy(ctx->instruction, ctx->mnemonic);
			break;
	}
	return ctx->instruction;
}
static void __dz80_default_on_before_exec(LPDZ80CONTEXT ctx) {}
static void __dz80_default_on_after_exec(LPDZ80CONTEXT ctx) {
#if _DEBUG
	if (1) {
		ctx->format_op(ctx);
		char buff[128];
		sprintf(buff, "[A:%2.2X F:%2.2X BC:%4.4X DE:%4.4X HL:%4.4X SP:%4.4X WZ:%4.4X]", ctx->cpu->r.h.A, ctx->cpu->r.h.F, ctx->cpu->r.x.BC, ctx->cpu->r.x.DE, ctx->cpu->r.x.HL, ctx->cpu->r.x.SP, ctx->cpu->r.x.WZ);

		if (ctx->cpu->id != 2)
			_RPT4(_CRT_WARN, "MAIN[%8.8X]: %4.4X: %s\t%s\n", ctx->cpu->extrn_sync_clk, ctx->PC, ctx->instruction, buff);
		else
			_RPT4(_CRT_WARN, "SUB [%8.8X]: %4.4X: %s\t%s\n", ctx->cpu->extrn_sync_clk, ctx->PC, ctx->instruction, buff);
	}
#endif
}
static void __dz80_default_on_req_signal(LPDZ80CONTEXT ctx) {}
static int  __dz80_default_on_query_halt_breakpoint(LPDZ80CONTEXT ctx) { return -1; }
static void __dz80_default_on_replace_breakpoint(LPDZ80CONTEXT ctx, int bpid) {}
static BOOL __dz80_default_on_restore_breakpoint(LPDZ80CONTEXT ctx, int bpid) { return FALSE; }
static void __dz80_default_on_remove_breakpoint(LPDZ80CONTEXT ctx, int bpid) {}
/**--------------------------------------------------------------------------
 **		Z80 Debug Context Initializer
 **------------------------------------------------------------------------*/
export void __dz80_set_external_method(LPZ80CPU cpu, int(__cdecl* z80_step)(LPZ80CPU), int(__cdecl* z80_int)(LPZ80CPU, int), void(__cdecl* z80_nmi)(LPZ80CPU)) {
	cpu->dbgctx.cpu					= cpu;
	cpu->dbgctx.z80_mbread			= cpu->mbread;			// save z80 original function pointer
	cpu->dbgctx.z80_mbwrite			= cpu->mbwrite;			// ditto
	cpu->dbgctx.z80_int_signal		= z80_int;				// ditto
	cpu->dbgctx.z80_nmi_signal		= z80_nmi;				// ditto
	cpu->dbgctx.z80_step			= z80_step;				// ditto
	cpu->dbgctx.z80_decoder			= cpu->decoder;			// ditto

	cpu->dbgctx.fetchb				= __dz80_fetchb;		// fetch for pre-decode (disasm)
	cpu->dbgctx.fetchb_M1			= __dz80_fetchb_M1;		// fetch M1 for pre-decode (disasm)
	cpu->dbgctx.pre_decode			= __dz80_pre_decoder;	// pre-decoder
	cpu->dbgctx.disasm				= __dz80_disasm;		// fetch from cpu memory
	cpu->dbgctx.format_op			= __dz80_format_op;
	cpu->dbgctx.on_before_exec		= __dz80_default_on_before_exec;
	cpu->dbgctx.on_after_exec		= __dz80_default_on_after_exec;
	cpu->dbgctx.on_req_signal		= __dz80_default_on_req_signal;
	cpu->dbgctx.on_query_breakpt	= __dz80_default_on_query_halt_breakpoint;
	cpu->dbgctx.on_replace_breakpt	= __dz80_default_on_replace_breakpoint;
	cpu->dbgctx.on_restore_breakpt	= __dz80_default_on_restore_breakpoint;
	cpu->dbgctx.on_remove_breakpt	= __dz80_default_on_remove_breakpoint;
	cpu->dbgctx.debug_enable		= __dz80_debug_enable;
	cpu->dbgctx.enable				= 1;
	__dz80_debug_enable(&cpu->dbgctx, 0);
}

#ifdef __cplusplus
}	// extern "C"
#endif
