/**--------------------------------------------------------------------------
 **
 **		Z80 Emulator Ver 3.00
 **		Copyright (C)1993-2010 H.Kanemoto
 **
 **		This is a free software. You may copy or distribute
 **		it without any permissions. But there are NO WARRANTY
 **		of use. Entire risk of use is with you.
 **
 **------------------------------------------------------------------------*/
#include "z80.h"
#include "z80mac.h"
#ifdef __cplusplus
extern "C"
{
#endif
#define export
/**--------------------------------------------------------------------------
 **		external functions
 **------------------------------------------------------------------------*/
extern void		_dz80_decoder_cb(Z80CPU *cpu);
extern void		_dz80_decoder_ed(Z80CPU *cpu);
extern void		_dz80_decoder_dd(Z80CPU *cpu);
extern void		_dz80_decoder_fd(Z80CPU* cpu);
/**--------------------------------------------------------------------------
 **		Decoder
 **------------------------------------------------------------------------*/
export void _dz80_dis_decoder(Z80CPU *cpu) {

	switch (cpu->dbgctx.fetchb_M1(cpu)) {
	case 0x00: __dz80_op_plane(cpu, "NOP");			break;
	case 0x01: __dz80_op_imm16(cpu, "LD BC,nn");	break;
	case 0x02: __dz80_op_plane(cpu, "LD (BC),A");	break;
	case 0x03: __dz80_op_plane(cpu, "INC BC");		break;
	case 0x04: __dz80_op_plane(cpu, "INC B");		break;
	case 0x05: __dz80_op_plane(cpu, "DEC B");		break;
	case 0x06: __dz80_op_imm8(cpu, "LD B,n");		break;
	case 0x07: __dz80_op_plane(cpu, "RLCA");		break;
	case 0x08: __dz80_op_plane(cpu, "EX AF,AF'");	break;
	case 0x09: __dz80_op_plane(cpu, "ADD HL,BC");	break;
	case 0x0A: __dz80_op_plane(cpu, "LD A,(BC)");	break;
	case 0x0B: __dz80_op_plane(cpu, "DEC BC");		break;
	case 0x0C: __dz80_op_plane(cpu, "INC C");		break;
	case 0x0D: __dz80_op_plane(cpu, "DEC C");		break;
	case 0x0E: __dz80_op_imm8(cpu, "LD C,n");		break;
	case 0x0F: __dz80_op_plane(cpu, "RRCA");		break;

	case 0x10: __dz80_op_offset(cpu, "DJNZ e");		break;
	case 0x11: __dz80_op_imm16(cpu, "LD DE,nn");	break;
	case 0x12: __dz80_op_plane(cpu, "LD (DE),A");	break;
	case 0x13: __dz80_op_plane(cpu, "INC DE");		break;
	case 0x14: __dz80_op_plane(cpu, "INC D");		break;
	case 0x15: __dz80_op_plane(cpu, "DEC D");		break;
	case 0x16: __dz80_op_imm8(cpu, "LD D,n");		break;
	case 0x17: __dz80_op_plane(cpu, "RLA");			break;
	case 0x18: __dz80_op_offset(cpu, "JR e");		break;
	case 0x19: __dz80_op_plane(cpu, "ADD HL,DE");	break;
	case 0x1A: __dz80_op_plane(cpu, "LD A,(DE)");	break;
	case 0x1B: __dz80_op_plane(cpu, "DEC DE");		break;
	case 0x1C: __dz80_op_plane(cpu, "INC E");		break;
	case 0x1D: __dz80_op_plane(cpu, "DEC E");		break;
	case 0x1E: __dz80_op_imm8(cpu, "LD E,n");		break;
	case 0x1F: __dz80_op_plane(cpu, "RRA");			break;

	case 0x20: __dz80_op_offset(cpu, "JR NZ,e");	break;
	case 0x21: __dz80_op_imm16(cpu, "LD HL,nn");	break;
	case 0x22: __dz80_op_mem16w(cpu, "LD (nn),HL"); break;
	case 0x23: __dz80_op_plane(cpu, "INC HL");		break;
	case 0x24: __dz80_op_plane(cpu, "INC H");		break;
	case 0x25: __dz80_op_plane(cpu, "DEC H");		break;
	case 0x26: __dz80_op_imm8(cpu, "LD H,n");		break;
	case 0x27: __dz80_op_plane(cpu, "DAA");			break;
	case 0x28: __dz80_op_offset(cpu, "JR Z,e");		break;
	case 0x29: __dz80_op_plane(cpu, "ADD HL,HL");	break;
	case 0x2A: __dz80_op_mem16r(cpu, "LD HL,(nn)"); break;
	case 0x2B: __dz80_op_plane(cpu, "DEC HL");		break;
	case 0x2C: __dz80_op_plane(cpu, "INC L");		break;
	case 0x2D: __dz80_op_plane(cpu, "DEC L");		break;
	case 0x2E: __dz80_op_imm8(cpu, "LD L,n");		break;
	case 0x2F: __dz80_op_plane(cpu, "CPL");			break;

	case 0x30: __dz80_op_offset(cpu, "JR NC,e");	break;
	case 0x31: __dz80_op_imm16(cpu, "LD SP,nn");	break;
	case 0x32: __dz80_op_mem8w(cpu, "LD (nn),A");	break;
	case 0x33: __dz80_op_plane(cpu, "INC SP");		break;
	case 0x34: __dz80_op_plane(cpu, "INC (HL)");	break;
	case 0x35: __dz80_op_plane(cpu, "DEC (HL)");	break;
	case 0x36: __dz80_op_imm8(cpu, "LD (HL),n");	break;
	case 0x37: __dz80_op_plane(cpu, "SCF");			break;
	case 0x38: __dz80_op_offset(cpu, "JR C,e");		break;
	case 0x39: __dz80_op_plane(cpu, "ADD HL,SP");	break;
	case 0x3A: __dz80_op_mem8r(cpu, "LD A,(nn)");	break;
	case 0x3B: __dz80_op_plane(cpu, "DEC SP");		break;
	case 0x3C: __dz80_op_plane(cpu, "INC A");		break;
	case 0x3D: __dz80_op_plane(cpu, "DEC A");		break;
	case 0x3E: __dz80_op_imm8(cpu, "LD A,n");		break;
	case 0x3F: __dz80_op_plane(cpu, "CCF");			break;

	case 0x40: __dz80_op_plane(cpu, "LD B,B");		break;
	case 0x41: __dz80_op_plane(cpu, "LD B,C");		break;
	case 0x42: __dz80_op_plane(cpu, "LD B,D");		break;
	case 0x43: __dz80_op_plane(cpu, "LD B,E");		break;
	case 0x44: __dz80_op_plane(cpu, "LD B,H");		break;
	case 0x45: __dz80_op_plane(cpu, "LD B,L");		break;
	case 0x46: __dz80_op_plane(cpu, "LD B,(HL)");	break;
	case 0x47: __dz80_op_plane(cpu, "LD B,A");		break;

	case 0x48: __dz80_op_plane(cpu, "LD C,B");		break;
	case 0x49: __dz80_op_plane(cpu, "LD C,C");		break;
	case 0x4A: __dz80_op_plane(cpu, "LD C,D");		break;
	case 0x4B: __dz80_op_plane(cpu, "LD C,E");		break;
	case 0x4C: __dz80_op_plane(cpu, "LD C,H");		break;
	case 0x4D: __dz80_op_plane(cpu, "LD C,L");		break;
	case 0x4E: __dz80_op_plane(cpu, "LD C,(HL)");	break;
	case 0x4F: __dz80_op_plane(cpu, "LD C,A");		break;

	case 0x50: __dz80_op_plane(cpu, "LD D,B");		break;
	case 0x51: __dz80_op_plane(cpu, "LD D,C");		break;
	case 0x52: __dz80_op_plane(cpu, "LD D,D");		break;
	case 0x53: __dz80_op_plane(cpu, "LD D,E");		break;
	case 0x54: __dz80_op_plane(cpu, "LD D,H");		break;
	case 0x55: __dz80_op_plane(cpu, "LD D,L");		break;
	case 0x56: __dz80_op_plane(cpu, "LD D,(HL)");	break;
	case 0x57: __dz80_op_plane(cpu, "LD D,A");		break;

	case 0x58: __dz80_op_plane(cpu, "LD E,B");		break;
	case 0x59: __dz80_op_plane(cpu, "LD E,C");		break;
	case 0x5A: __dz80_op_plane(cpu, "LD E,D");		break;
	case 0x5B: __dz80_op_plane(cpu, "LD E,E");		break;
	case 0x5C: __dz80_op_plane(cpu, "LD E,H");		break;
	case 0x5D: __dz80_op_plane(cpu, "LD E,L");		break;
	case 0x5E: __dz80_op_plane(cpu, "LD E,(HL)");	break;
	case 0x5F: __dz80_op_plane(cpu, "LD E,A");		break;

	case 0x60: __dz80_op_plane(cpu, "LD H,B");		break;
	case 0x61: __dz80_op_plane(cpu, "LD H,C");		break;
	case 0x62: __dz80_op_plane(cpu, "LD H,D");		break;
	case 0x63: __dz80_op_plane(cpu, "LD H,E");		break;
	case 0x64: __dz80_op_plane(cpu, "LD H,H");		break;
	case 0x65: __dz80_op_plane(cpu, "LD H,L");		break;
	case 0x66: __dz80_op_plane(cpu, "LD H,(HL)");	break;
	case 0x67: __dz80_op_plane(cpu, "LD H,A");		break;

	case 0x68: __dz80_op_plane(cpu, "LD L,B");		break;
	case 0x69: __dz80_op_plane(cpu, "LD L,C");		break;
	case 0x6A: __dz80_op_plane(cpu, "LD L,D");		break;
	case 0x6B: __dz80_op_plane(cpu, "LD L,E");		break;
	case 0x6C: __dz80_op_plane(cpu, "LD L,H");		break;
	case 0x6D: __dz80_op_plane(cpu, "LD L,L");		break;
	case 0x6E: __dz80_op_plane(cpu, "LD L,(HL)");	break;
	case 0x6F: __dz80_op_plane(cpu, "LD L,A");		break;

	case 0x70: __dz80_op_plane(cpu, "LD (HL),B");	break;
	case 0x71: __dz80_op_plane(cpu, "LD (HL),C");	break;
	case 0x72: __dz80_op_plane(cpu, "LD (HL),D");	break;
	case 0x73: __dz80_op_plane(cpu, "LD (HL),E");	break;
	case 0x74: __dz80_op_plane(cpu, "LD (HL),H");	break;
	case 0x75: __dz80_op_plane(cpu, "LD (HL),L");	break;
	case 0x76: __dz80_op_plane(cpu, "HALT");		break;
	case 0x77: __dz80_op_plane(cpu, "LD (HL),A");	break;

	case 0x78: __dz80_op_plane(cpu, "LD A,B");		break;
	case 0x79: __dz80_op_plane(cpu, "LD A,C");		break;
	case 0x7A: __dz80_op_plane(cpu, "LD A,D");		break;
	case 0x7B: __dz80_op_plane(cpu, "LD A,E");		break;
	case 0x7C: __dz80_op_plane(cpu, "LD A,H");		break;
	case 0x7D: __dz80_op_plane(cpu, "LD A,L");		break;
	case 0x7E: __dz80_op_plane(cpu, "LD A,(HL)");	break;
	case 0x7F: __dz80_op_plane(cpu, "LD A,A");		break;

	case 0x80: __dz80_op_plane(cpu, "ADD A,B");		break;
	case 0x81: __dz80_op_plane(cpu, "ADD A,C");		break;
	case 0x82: __dz80_op_plane(cpu, "ADD A,D");		break;
	case 0x83: __dz80_op_plane(cpu, "ADD A,E");		break;
	case 0x84: __dz80_op_plane(cpu, "ADD A,H");		break;
	case 0x85: __dz80_op_plane(cpu, "ADD A,L");		break;
	case 0x86: __dz80_op_plane(cpu, "ADD A,(HL)");	break;
	case 0x87: __dz80_op_plane(cpu, "ADD A,A");		break;

	case 0x88: __dz80_op_plane(cpu, "ADC A,B");		break;
	case 0x89: __dz80_op_plane(cpu, "ADC A,C");		break;
	case 0x8A: __dz80_op_plane(cpu, "ADC A,D");		break;
	case 0x8B: __dz80_op_plane(cpu, "ADC A,E");		break;
	case 0x8C: __dz80_op_plane(cpu, "ADC A,H");		break;
	case 0x8D: __dz80_op_plane(cpu, "ADC A,L");		break;
	case 0x8E: __dz80_op_plane(cpu, "ADC A,(HL)");	break;
	case 0x8F: __dz80_op_plane(cpu, "ADC A,A");		break;

	case 0x90: __dz80_op_plane(cpu, "SUB A,B");		break;
	case 0x91: __dz80_op_plane(cpu, "SUB A,C");		break;
	case 0x92: __dz80_op_plane(cpu, "SUB A,D");		break;
	case 0x93: __dz80_op_plane(cpu, "SUB A,E");		break;
	case 0x94: __dz80_op_plane(cpu, "SUB A,H");		break;
	case 0x95: __dz80_op_plane(cpu, "SUB A,L");		break;
	case 0x96: __dz80_op_plane(cpu, "SUB A,(HL)");	break;
	case 0x97: __dz80_op_plane(cpu, "SUB A,A");		break;

	case 0x98: __dz80_op_plane(cpu, "SBC A,B");		break;
	case 0x99: __dz80_op_plane(cpu, "SBC A,C");		break;
	case 0x9A: __dz80_op_plane(cpu, "SBC A,D");		break;
	case 0x9B: __dz80_op_plane(cpu, "SBC A,E");		break;
	case 0x9C: __dz80_op_plane(cpu, "SBC A,H");		break;
	case 0x9D: __dz80_op_plane(cpu, "SBC A,L");		break;
	case 0x9E: __dz80_op_plane(cpu, "SBC A,(HL)");	break;
	case 0x9F: __dz80_op_plane(cpu, "SBC A,A");		break;

	case 0xA0: __dz80_op_plane(cpu, "AND A,B");		break;
	case 0xA1: __dz80_op_plane(cpu, "AND A,C");		break;
	case 0xA2: __dz80_op_plane(cpu, "AND A,D");		break;
	case 0xA3: __dz80_op_plane(cpu, "AND A,E");		break;
	case 0xA4: __dz80_op_plane(cpu, "AND A,H");		break;
	case 0xA5: __dz80_op_plane(cpu, "AND A,L");		break;
	case 0xA6: __dz80_op_plane(cpu, "AND A,(HL)");	break;
	case 0xA7: __dz80_op_plane(cpu, "AND A,A");		break;

	case 0xA8: __dz80_op_plane(cpu, "XOR A,B");		break;
	case 0xA9: __dz80_op_plane(cpu, "XOR A,C");		break;
	case 0xAA: __dz80_op_plane(cpu, "XOR A,D");		break;
	case 0xAB: __dz80_op_plane(cpu, "XOR A,E");		break;
	case 0xAC: __dz80_op_plane(cpu, "XOR A,H");		break;
	case 0xAD: __dz80_op_plane(cpu, "XOR A,L");		break;
	case 0xAE: __dz80_op_plane(cpu, "XOR A,(HL)");	break;
	case 0xAF: __dz80_op_plane(cpu, "XOR A,A");		break;

	case 0xB0: __dz80_op_plane(cpu, "OR A,B");		break;
	case 0xB1: __dz80_op_plane(cpu, "OR A,C");		break;
	case 0xB2: __dz80_op_plane(cpu, "OR A,D");		break;
	case 0xB3: __dz80_op_plane(cpu, "OR A,E");		break;
	case 0xB4: __dz80_op_plane(cpu, "OR A,H");		break;
	case 0xB5: __dz80_op_plane(cpu, "OR A,L");		break;
	case 0xB6: __dz80_op_plane(cpu, "OR A,(HL) ");	break;
	case 0xB7: __dz80_op_plane(cpu, "OR A,A");		break;

	case 0xB8: __dz80_op_plane(cpu, "CP A,B");		break;
	case 0xB9: __dz80_op_plane(cpu, "CP A,C");		break;
	case 0xBA: __dz80_op_plane(cpu, "CP A,D");		break;
	case 0xBB: __dz80_op_plane(cpu, "CP A,E");		break;
	case 0xBC: __dz80_op_plane(cpu, "CP A,H");		break;
	case 0xBD: __dz80_op_plane(cpu, "CP A,L");		break;
	case 0xBE: __dz80_op_plane(cpu, "CP A,(HL)");	break;
	case 0xBF: __dz80_op_plane(cpu, "CP A,A");		break;

	case 0xC0: __dz80_op_plane(cpu, "RET NZ");		break;
	case 0xC1: __dz80_op_plane(cpu, "POP BC");		break;
	case 0xC2: __dz80_op_imm16(cpu, "JP NZ,nn");	break;
	case 0xC3: __dz80_op_imm16(cpu, "JP nn");		break;
	case 0xC4: __dz80_op_imm16(cpu, "CALL NZ,nn");	break;
	case 0xC5: __dz80_op_plane(cpu, "PUSH BC");		break;
	case 0xC6: __dz80_op_imm8(cpu, "ADD A,n");		break;
	case 0xC7: __dz80_op_plane(cpu, "RST 00H");		break;

	case 0xC8: __dz80_op_plane(cpu, "RET Z");		break;
	case 0xC9: __dz80_op_plane(cpu, "RET");			break;
	case 0xCA: __dz80_op_imm16(cpu, "JP Z,nn");		break;
	case 0xCB:	_dz80_decoder_cb(cpu);				break;
	case 0xCC: __dz80_op_imm16(cpu, "CALL Z,nn");	break;
	case 0xCD: __dz80_op_imm16(cpu, "CALL nn");		break;
	case 0xCE: __dz80_op_imm8(cpu, "ADC A,n");		break;
	case 0xCF: __dz80_op_plane(cpu, "RST 08H");		break;

	case 0xD0: __dz80_op_plane(cpu, "RET NC");		break;
	case 0xD1: __dz80_op_plane(cpu, "POP DE");		break;
	case 0xD2: __dz80_op_imm16(cpu, "JP NC,nn");	break;
	case 0xD3: __dz80_op_imm8(cpu, "OUT (n),A");	break;
	case 0xD4: __dz80_op_imm16(cpu, "CALL NC,nn ");	break;
	case 0xD5: __dz80_op_plane(cpu, "PUSH DE");		break;
	case 0xD6: __dz80_op_imm8(cpu, "SUB A,n");		break;
	case 0xD7: __dz80_op_plane(cpu, "RST 10H");		break;

	case 0xD8: __dz80_op_plane(cpu, "RET C");		break;
	case 0xD9: __dz80_op_plane(cpu, "EXX");			break;
	case 0xDA: __dz80_op_imm16(cpu, "JP C,nn");		break;
	case 0xDB: __dz80_op_imm8(cpu, "IN A,(n)");		break;
	case 0xDC: __dz80_op_imm16(cpu, "CALL C,nn");	break;
	case 0xDD:	_dz80_decoder_dd(cpu);				break;
	case 0xDE: __dz80_op_imm8(cpu, "SBC A,n");		break;
	case 0xDF: __dz80_op_plane(cpu, "RST 18H");	break;

	case 0xE0: __dz80_op_plane(cpu, "RET PO");		break;
	case 0xE1: __dz80_op_plane(cpu, "POP HL");		break;
	case 0xE2: __dz80_op_imm16(cpu, "JP PO,nn");	break;
	case 0xE3: __dz80_op_plane(cpu, "EX (SP),HL");	break;
	case 0xE4: __dz80_op_imm16(cpu, "CALL PO,nn");	break;
	case 0xE5: __dz80_op_plane(cpu, "PUSH HL");		break;
	case 0xE6: __dz80_op_imm8(cpu, "AND A,n");		break;
	case 0xE7: __dz80_op_plane(cpu, "RST 20H");		break;

	case 0xE8: __dz80_op_plane(cpu, "RET PE");		break;
	case 0xE9: __dz80_op_plane(cpu, "JP (HL)");		break;
	case 0xEA: __dz80_op_imm16(cpu, "JP PE,nn");	break;
	case 0xEB: __dz80_op_plane(cpu, "EX DE,HL");	break;
	case 0xEC: __dz80_op_imm16(cpu, "CALL PC,nn");	break;
	case 0xED:	_dz80_decoder_ed(cpu); 				break;
	case 0xEE: __dz80_op_imm8(cpu, "XOR A,n");		break;
	case 0xEF: __dz80_op_plane(cpu, "RST 28H");		break;

	case 0xF0: __dz80_op_plane(cpu, "RET P");		break;
	case 0xF1: __dz80_op_plane(cpu, "POP AF");		break;
	case 0xF2: __dz80_op_imm16(cpu, "JP P,nn");		break;
	case 0xF3: __dz80_op_plane(cpu, "DI");			break;
	case 0xF4: __dz80_op_imm16(cpu, "CALL P,nn");	break;
	case 0xF5: __dz80_op_plane(cpu, "PUSH AF");		break;
	case 0xF6: __dz80_op_imm8(cpu, "OR A,n");		break;
	case 0xF7: __dz80_op_plane(cpu, "RST 30H");		break;

	case 0xF8: __dz80_op_plane(cpu, "RET M");		break;
	case 0xF9: __dz80_op_plane(cpu, "LD SP,HL");	break;
	case 0xFA: __dz80_op_imm16(cpu, "JP M,nn");		break;
	case 0xFB: __dz80_op_plane(cpu, "EI");			break;
	case 0xFC: __dz80_op_imm16(cpu, "CALL M,nn ");	break;
	case 0xFD:	_dz80_decoder_fd(cpu);				break;
	case 0xFE: __dz80_op_imm8(cpu, "CP A,n");		break;
	case 0xFF: __dz80_op_plane(cpu, "RST 38H");		break;
	}
}

#ifdef __cplusplus
}	// extern "C"
#endif
