/**--------------------------------------------------------------------------
 **
 **		Z80 Emulator Ver 3.00
 **		Copyright (C)1993-2010 H.Kanemoto
 **
 **		This is a free software. You may copy or distribute
 **		it without any permissions. But there are NO WARRANTY
 **		of use. Entire risk of use is with you.
 **
 **------------------------------------------------------------------------*/
#include "z80.h"
#include "z80mac.h"
#ifdef __cplusplus
extern "C"
{
#endif
#define export
/**--------------------------------------------------------------------------
 **		ASM Modules
 **------------------------------------------------------------------------*/
extern void _dz80_decoder_dd_cb(Z80CPU *cpu);
extern void _dz80_decoder_fd_cb(Z80CPU *cpu);
/**--------------------------------------------------------------------------
 **		Decoder
 **------------------------------------------------------------------------*/
export void _dz80_decoder_dd(Z80CPU *cpu) {
	switch (cpu->dbgctx.fetchb(cpu)) {
	case 0x09: __dz80_op_plane(cpu, "ADD IX,BC");		break;

	case 0x19: __dz80_op_plane(cpu, "ADD IX,DE");		break;

	case 0x21: __dz80_op_imm16(cpu, "LD,IX,nn");		break;
	case 0x22: __dz80_op_mem16w(cpu, "LD (nn).IX");		break;
	case 0x23: __dz80_op_plane(cpu, "INC IX");			break;
	case 0x24: __dz80_op_plane(cpu, "INC IXH");			break;
	case 0x25: __dz80_op_plane(cpu, "DEC IXH");			break;
	case 0x26: __dz80_op_imm8(cpu, "LD IXH,n");			break;
	case 0x29: __dz80_op_plane(cpu, "ADD IX,HL");		break;
	case 0x2A: __dz80_op_mem16r(cpu, "LD IX,(nn)");		break;
	case 0x2B: __dz80_op_plane(cpu, "DEC IX");			break;
	case 0x2C: __dz80_op_plane(cpu, "INC IXL");			break;
	case 0x2D: __dz80_op_plane(cpu, "DEC IXL");			break;
	case 0x2E: __dz80_op_imm8(cpu, "LD IXL,n");			break;

	case 0x34: __dz80_op_ixd(cpu, "INC (IX+d)");		break;
	case 0x35: __dz80_op_ixd(cpu, "DEC (IX+d)");		break;
	case 0x36: __dz80_op_ixd_imm8(cpu, "LD (IX+d),n");	break;
	case 0x39: __dz80_op_plane(cpu, "ADD IX,SP");		break;

	case 0x40: __dz80_op_plane(cpu, "LD B,B");			break;	// shadow
	case 0x41: __dz80_op_plane(cpu, "LD B,C");			break;	// shadow
	case 0x42: __dz80_op_plane(cpu, "LD B,D");			break;	// shadow
	case 0x43: __dz80_op_plane(cpu, "LD B,E");			break;	// shadow
	case 0x44: __dz80_op_plane(cpu, "LD B,IXH");		break;
	case 0x45: __dz80_op_plane(cpu, "LD B,IXL");		break;
	case 0x46: __dz80_op_ixd(cpu, "LD B,(IX+d)");		break;
	case 0x47: __dz80_op_plane(cpu, "LD B,A");			break;	// shadow

	case 0x48: __dz80_op_plane(cpu, "LD C,B");			break;	// shadow
	case 0x49: __dz80_op_plane(cpu, "LD C,C");			break;	// shadow
	case 0x4A: __dz80_op_plane(cpu, "LD C,D");			break;	// shadow
	case 0x4B: __dz80_op_plane(cpu, "LD C,E");			break;	// shadow
	case 0x4C: __dz80_op_plane(cpu, "LD C,IXH");		break;
	case 0x4D: __dz80_op_plane(cpu, "LD C,IXL");		break;
	case 0x4E: __dz80_op_ixd(cpu, "LD C,(IX+d)");		break;
	case 0x4F: __dz80_op_plane(cpu, "LD C,A");			break;	// shadow

	case 0x50: __dz80_op_plane(cpu, "LD D,B");			break;	// shadow
	case 0x51: __dz80_op_plane(cpu, "LD D,C");			break;	// shadow
	case 0x52: __dz80_op_plane(cpu, "LD D,D");			break;	// shadow
	case 0x53: __dz80_op_plane(cpu, "LD D,E");			break;	// shadow
	case 0x54: __dz80_op_plane(cpu, "LD D,IXH");		break;
	case 0x55: __dz80_op_plane(cpu, "LD D,IXL");		break;
	case 0x56: __dz80_op_ixd(cpu, "LD D,(IX+d)");		break;
	case 0x57: __dz80_op_plane(cpu, "LD D,A");			break;	// shadow

	case 0x58: __dz80_op_plane(cpu, "LD E,B");			break;	// shadow
	case 0x59: __dz80_op_plane(cpu, "LD E,C");			break;	// shadow
	case 0x5A: __dz80_op_plane(cpu, "LD E,D");			break;	// shadow
	case 0x5B: __dz80_op_plane(cpu, "LD E,E");			break;	// shadow
	case 0x5C: __dz80_op_plane(cpu, "LD E,IXH");		break;
	case 0x5D: __dz80_op_plane(cpu, "LD E,IXL");		break;
	case 0x5E: __dz80_op_ixd(cpu, "LD E,(IX+d)");		break;
	case 0x5F: __dz80_op_plane(cpu, "LD E,A");			break;	// shadow

	case 0x60: __dz80_op_plane(cpu, "LD IXH,B");		break;
	case 0x61: __dz80_op_plane(cpu, "LD IXH,C");		break;
	case 0x62: __dz80_op_plane(cpu, "LD IXH,D");		break;
	case 0x63: __dz80_op_plane(cpu, "LD IXH,E");		break;
	case 0x64: __dz80_op_plane(cpu, "LD IXH,IXH");		break;
	case 0x65: __dz80_op_plane(cpu, "LD IXH,IXL");		break;
	case 0x66: __dz80_op_ixd(cpu, "LD H,(IX+d)");		break;
	case 0x67: __dz80_op_plane(cpu, "LD IXH,A");		break;

	case 0x68: __dz80_op_plane(cpu, "LD IXL,B");		break;
	case 0x69: __dz80_op_plane(cpu, "LD IXL,C");		break;
	case 0x6A: __dz80_op_plane(cpu, "LD IXL,D");		break;
	case 0x6B: __dz80_op_plane(cpu, "LD IXL,E");		break;
	case 0x6C: __dz80_op_plane(cpu, "LD IXL,IXH");		break;
	case 0x6D: __dz80_op_plane(cpu, "LD IXL,IXL");		break;
	case 0x6E: __dz80_op_ixd(cpu, "LD L,(IX+d)");		break;
	case 0x6F: __dz80_op_plane(cpu, "LD IXL,A");		break;

	case 0x70: __dz80_op_ixd(cpu, "LD (IX+d),B");		break;
	case 0x71: __dz80_op_ixd(cpu, "LD (IX+d),C");		break;
	case 0x72: __dz80_op_ixd(cpu, "LD (IX+d),D");		break;
	case 0x73: __dz80_op_ixd(cpu, "LD (IX+d),E");		break;
	case 0x74: __dz80_op_ixd(cpu, "LD (IX+d),H");		break;
	case 0x75: __dz80_op_ixd(cpu, "LD (IX+d),L");		break;
	case 0x77: __dz80_op_ixd(cpu, "LD (IX+d),A");		break;

	case 0x78: __dz80_op_plane(cpu, "LD A,B");			break;	// shadow
	case 0x79: __dz80_op_plane(cpu, "LD A,C");			break;	// shadow
	case 0x7A: __dz80_op_plane(cpu, "LD A,D");			break;	// shadow
	case 0x7B: __dz80_op_plane(cpu, "LD A,E");			break;	// shadow
	case 0x7C: __dz80_op_plane(cpu, "LD A,IXH");		break;
	case 0x7D: __dz80_op_plane(cpu, "LD A,IXL");		break;
	case 0x7E: __dz80_op_ixd(cpu, "LD A,(IX+d)");		break;
	case 0x7F: __dz80_op_plane(cpu, "LD A,A");			break;	// shadow

	case 0x80: __dz80_op_plane(cpu, "ADD A,B");			break;	// shadow
	case 0x81: __dz80_op_plane(cpu, "ADD A,C");			break;	// shadow
	case 0x82: __dz80_op_plane(cpu, "ADD A,D");			break;	// shadow
	case 0x83: __dz80_op_plane(cpu, "ADD A,E");			break;	// shadow
	case 0x84: __dz80_op_plane(cpu, "ADD A,IXH");		break;
	case 0x85: __dz80_op_plane(cpu, "ADD A,IXL");		break;
	case 0x86: __dz80_op_ixd(cpu, "ADD A,(IX+d)");		break;
	case 0x87: __dz80_op_plane(cpu, "ADD A,A");			break;	// shadow

	case 0x88: __dz80_op_plane(cpu, "ADC A,B");			break;	// shadow
	case 0x89: __dz80_op_plane(cpu, "ADC A,C");			break;	// shadow
	case 0x8A: __dz80_op_plane(cpu, "ADC A,D");			break;	// shadow
	case 0x8B: __dz80_op_plane(cpu, "ADC A,E");			break;	// shadow
	case 0x8C: __dz80_op_plane(cpu, "ADC A,IXH");		break;
	case 0x8D: __dz80_op_plane(cpu, "ADC A,IXL");		break;
	case 0x8E: __dz80_op_ixd(cpu, "ADC A,(IX+d)");		break;
	case 0x8F: __dz80_op_plane(cpu, "ADC A,A");			break;	// shadow

	case 0x90: __dz80_op_plane(cpu, "SUB A,B");			break;	// shadow
	case 0x91: __dz80_op_plane(cpu, "SUB A,C");			break;	// shadow
	case 0x92: __dz80_op_plane(cpu, "SUB A,D");			break;	// shadow
	case 0x93: __dz80_op_plane(cpu, "SUB A,E");			break;	// shadow
	case 0x94: __dz80_op_plane(cpu, "SUB A,IXH");		break;
	case 0x95: __dz80_op_plane(cpu, "SUB A,IXL");		break;
	case 0x96: __dz80_op_ixd(cpu, "SUB A,(IX+d)");		break;
	case 0x97: __dz80_op_plane(cpu, "SUB A,A");			break;	// shadow

	case 0x98: __dz80_op_plane(cpu, "SBC A,B");			break;	// shadow
	case 0x99: __dz80_op_plane(cpu, "SBC A,C");			break;	// shadow
	case 0x9A: __dz80_op_plane(cpu, "SBC A,D");			break;	// shadow
	case 0x9B: __dz80_op_plane(cpu, "SBC A,E");			break;	// shadow
	case 0x9C: __dz80_op_plane(cpu, "SBC A,IXH");		break;
	case 0x9D: __dz80_op_plane(cpu, "SBC A,IXL");		break;
	case 0x9E: __dz80_op_ixd(cpu, "SBC A,(IX+d)");		break;
	case 0x9F: __dz80_op_plane(cpu, "SBC A,A");			break;	// shadow

	case 0xA0: __dz80_op_plane(cpu, "AND A,B");			break;	// shadow
	case 0xA1: __dz80_op_plane(cpu, "AND A,C");			break;	// shadow
	case 0xA2: __dz80_op_plane(cpu, "AND A,D");			break;	// shadow
	case 0xA3: __dz80_op_plane(cpu, "AND A,E");			break;	// shadow
	case 0xA4: __dz80_op_plane(cpu, "AND A,IXH");		break;
	case 0xA5: __dz80_op_plane(cpu, "AND A,IXL");		break;
	case 0xA6: __dz80_op_ixd(cpu, "AND A,(IX+d)");		break;
	case 0xA7: __dz80_op_plane(cpu, "AND A,A");			break;	// shadow

	case 0xA8: __dz80_op_plane(cpu, "XOR A,B");			break;	// shadow
	case 0xA9: __dz80_op_plane(cpu, "XOR A,C");			break;	// shadow
	case 0xAA: __dz80_op_plane(cpu, "XOR A,D");			break;	// shadow
	case 0xAB: __dz80_op_plane(cpu, "XOR A,E");			break;	// shadow
	case 0xAC: __dz80_op_plane(cpu, "XOR A,IXH");		break;
	case 0xAD: __dz80_op_plane(cpu, "XOR A,IXL");		break;
	case 0xAE: __dz80_op_ixd(cpu, "XOR A,(IX+d)");		break;
	case 0xAF: __dz80_op_plane(cpu, "XOR A,A");			break;	// shadow

	case 0xB0: __dz80_op_plane(cpu, "OR A,B");			break;	// shadow
	case 0xB1: __dz80_op_plane(cpu, "OR A,C");			break;	// shadow
	case 0xB2: __dz80_op_plane(cpu, "OR A,D");			break;	// shadow
	case 0xB3: __dz80_op_plane(cpu, "OR A,E");			break;	// shadow
	case 0xB4: __dz80_op_plane(cpu, "OR A,IXH");		break;
	case 0xB5: __dz80_op_plane(cpu, "OR A,IXL");		break;
	case 0xB6: __dz80_op_ixd(cpu, "OR A,(IX+d)");		break;
	case 0xB7: __dz80_op_plane(cpu, "OR A,A");			break;	// shadow

	case 0xB8: __dz80_op_plane(cpu, "CP A,B");			break;	// shadow
	case 0xB9: __dz80_op_plane(cpu, "CP A,C");			break;	// shadow
	case 0xBA: __dz80_op_plane(cpu, "CP A,D");			break;	// shadow
	case 0xBB: __dz80_op_plane(cpu, "CP A,E");			break;	// shadow
	case 0xBC: __dz80_op_plane(cpu, "CP IXH");			break;
	case 0xBD: __dz80_op_plane(cpu, "CP IXL");			break;
	case 0xBE: __dz80_op_ixd(cpu, "CP (IX+d)");			break;
	case 0xBF: __dz80_op_plane(cpu, "CP A,A");			break;	// shadow

	case 0xCB:	_dz80_decoder_dd_cb(cpu); 				break;

	case 0xE1: __dz80_op_plane(cpu, "POP IX");			break;
	case 0xE3: __dz80_op_plane(cpu, "EX (SP),IX");		break;
	case 0xE5: __dz80_op_plane(cpu, "PUSH IX");			break;
	case 0xE9: __dz80_op_plane(cpu, "JP (IX)");			break;

	case 0xF9: __dz80_op_plane(cpu, "LD SP,IX");		break;
	default: break;
	}
}
export void _dz80_decoder_fd(Z80CPU *cpu) {
	switch (cpu->dbgctx.fetchb(cpu)) {
	case 0x09: __dz80_op_plane(cpu, "ADD IY,BC");		break;
	case 0x19: __dz80_op_plane(cpu, "ADD IY,DE");		break;
	case 0x21: __dz80_op_imm16(cpu, "LD,IY,nn");		break;
	case 0x22: __dz80_op_mem16w(cpu, "LD (nn).IY");		break;
	case 0x23: __dz80_op_plane(cpu, "INC IY");			break;
	case 0x24: __dz80_op_plane(cpu, "INC IYH");			break;
	case 0x25: __dz80_op_plane(cpu, "DEC IYH");			break;
	case 0x26: __dz80_op_imm8(cpu, "LD IYH,n");			break;
	case 0x29: __dz80_op_plane(cpu, "ADD IY,HL");		break;
	case 0x2A: __dz80_op_mem16r(cpu, "LD IY,(nn)");		break;
	case 0x2B: __dz80_op_plane(cpu, "DEC IY");			break;
	case 0x2C: __dz80_op_plane(cpu, "INC IYL");			break;
	case 0x2D: __dz80_op_plane(cpu, "DEC IYL");			break;
	case 0x2E: __dz80_op_imm8(cpu, "LD IYL,n");			break;
	case 0x34: __dz80_op_ixd(cpu, "INC (IY+d)");		break;
	case 0x35: __dz80_op_ixd(cpu, "DEC (IY+d)");		break;
	case 0x36: __dz80_op_ixd_imm8(cpu, "LD (IY+d),n");	break;
	case 0x39: __dz80_op_plane(cpu, "ADD IY,SP");		break;

	case 0x40: __dz80_op_plane(cpu, "LD B,B");			break;	// shadow
	case 0x41: __dz80_op_plane(cpu, "LD B,C");			break;	// shadow
	case 0x42: __dz80_op_plane(cpu, "LD B,D");			break;	// shadow
	case 0x43: __dz80_op_plane(cpu, "LD B,E");			break;	// shadow
	case 0x44: __dz80_op_plane(cpu, "LD B,IYH");		break;
	case 0x45: __dz80_op_plane(cpu, "LD B,IYL");		break;
	case 0x46: __dz80_op_ixd(cpu, "LD B,(IY+d)");		break;
	case 0x47: __dz80_op_plane(cpu, "LD B,A");			break;	// shadow

	case 0x48: __dz80_op_plane(cpu, "LD C,B");			break;	// shadow
	case 0x49: __dz80_op_plane(cpu, "LD C,C");			break;	// shadow
	case 0x4A: __dz80_op_plane(cpu, "LD C,D");			break;	// shadow
	case 0x4B: __dz80_op_plane(cpu, "LD C,E");			break;	// shadow
	case 0x4C: __dz80_op_plane(cpu, "LD C,IYH");		break;
	case 0x4D: __dz80_op_plane(cpu, "LD C,IYL");		break;
	case 0x4E: __dz80_op_ixd(cpu, "LD C,(IY+d)");		break;
	case 0x4F: __dz80_op_plane(cpu, "LD C,A");			break;	// shadow

	case 0x50: __dz80_op_plane(cpu, "LD D,B");			break;	// shadow
	case 0x51: __dz80_op_plane(cpu, "LD D,C");			break;	// shadow
	case 0x52: __dz80_op_plane(cpu, "LD D,D");			break;	// shadow
	case 0x53: __dz80_op_plane(cpu, "LD D,E");			break;	// shadow
	case 0x54: __dz80_op_plane(cpu, "LD D,IYH");		break;
	case 0x55: __dz80_op_plane(cpu, "LD D,IYL");		break;
	case 0x56: __dz80_op_ixd(cpu, "LD D,(IY+d)");		break;
	case 0x57: __dz80_op_plane(cpu, "LD D,A");			break;	// shadow

	case 0x58: __dz80_op_plane(cpu, "LD E,B");			break;	// shadow
	case 0x59: __dz80_op_plane(cpu, "LD E,C");			break;	// shadow
	case 0x5A: __dz80_op_plane(cpu, "LD E,D");			break;	// shadow
	case 0x5B: __dz80_op_plane(cpu, "LD E,E");			break;	// shadow
	case 0x5C: __dz80_op_plane(cpu, "LD E,IYH");		break;
	case 0x5D: __dz80_op_plane(cpu, "LD E,IYL");		break;
	case 0x5E: __dz80_op_ixd(cpu, "LD E,(IY+d)");		break;
	case 0x5F: __dz80_op_plane(cpu, "LD E,A");			break;	// shadow

	case 0x60: __dz80_op_plane(cpu, "LD IYH,B");		break;
	case 0x61: __dz80_op_plane(cpu, "LD IYH,C");		break;
	case 0x62: __dz80_op_plane(cpu, "LD IYH,D");		break;
	case 0x63: __dz80_op_plane(cpu, "LD IYH,E");		break;
	case 0x64: __dz80_op_plane(cpu, "LD IYH,IYH");		break;
	case 0x65: __dz80_op_plane(cpu, "LD IYH,IYL");		break;
	case 0x66: __dz80_op_ixd(cpu, "LD H,(IY+d)");		break;
	case 0x67: __dz80_op_plane(cpu, "LD IYH,A");		break;

	case 0x68: __dz80_op_plane(cpu, "LD IYL,B");		break;
	case 0x69: __dz80_op_plane(cpu, "LD IYL,C");		break;
	case 0x6A: __dz80_op_plane(cpu, "LD IYL,D");		break;
	case 0x6B: __dz80_op_plane(cpu, "LD IYL,E");		break;
	case 0x6C: __dz80_op_plane(cpu, "LD IYL,IYH");		break;
	case 0x6D: __dz80_op_plane(cpu, "LD IYL,IYL");		break;
	case 0x6E: __dz80_op_ixd(cpu, "LD L,(IY+d)");		break;
	case 0x6F: __dz80_op_plane(cpu, "LD IYL,A");		break;

	case 0x70: __dz80_op_ixd(cpu, "LD (IY+d),B");		break;
	case 0x71: __dz80_op_ixd(cpu, "LD (IY+d),C");		break;
	case 0x72: __dz80_op_ixd(cpu, "LD (IY+d),D");		break;
	case 0x73: __dz80_op_ixd(cpu, "LD (IY+d),E");		break;
	case 0x74: __dz80_op_ixd(cpu, "LD (IY+d),H");		break;
	case 0x75: __dz80_op_ixd(cpu, "LD (IY+d),L");		break;
	case 0x77: __dz80_op_ixd(cpu, "LD (IY+d),A");		break;

	case 0x78: __dz80_op_plane(cpu, "LD A,B");			break;	// shadow
	case 0x79: __dz80_op_plane(cpu, "LD A,C");			break;	// shadow
	case 0x7A: __dz80_op_plane(cpu, "LD A,D");			break;	// shadow
	case 0x7B: __dz80_op_plane(cpu, "LD A,E");			break;	// shadow
	case 0x7C: __dz80_op_plane(cpu, "LD A,IYH");		break;
	case 0x7D: __dz80_op_plane(cpu, "LD A,IYL");		break;
	case 0x7E: __dz80_op_ixd(cpu, "LD A,(IY+d)");		break;
	case 0x7F: __dz80_op_plane(cpu, "LD A,A");			break;	// shadow

	case 0x80: __dz80_op_plane(cpu, "ADD A,B");			break;	// shadow
	case 0x81: __dz80_op_plane(cpu, "ADD A,C");			break;	// shadow
	case 0x82: __dz80_op_plane(cpu, "ADD A,D");			break;	// shadow
	case 0x83: __dz80_op_plane(cpu, "ADD A,E");			break;	// shadow
	case 0x84: __dz80_op_plane(cpu, "ADD A,IYH");		break;
	case 0x85: __dz80_op_plane(cpu, "ADD A,IYL");		break;
	case 0x86: __dz80_op_ixd(cpu, "ADD A,(IY+d)");		break;
	case 0x87: __dz80_op_plane(cpu, "ADD A,A");			break;	// shadow

	case 0x88: __dz80_op_plane(cpu, "ADC A,B");			break;	// shadow
	case 0x89: __dz80_op_plane(cpu, "ADC A,C");			break;	// shadow
	case 0x8A: __dz80_op_plane(cpu, "ADC A,D");			break;	// shadow
	case 0x8B: __dz80_op_plane(cpu, "ADC A,E");			break;	// shadow
	case 0x8C: __dz80_op_plane(cpu, "ADC A,IYH");		break;
	case 0x8D: __dz80_op_plane(cpu, "ADC A,IYL");		break;
	case 0x8E: __dz80_op_ixd(cpu, "ADC A,(IY+d)");		break;
	case 0x8F: __dz80_op_plane(cpu, "ADC A,A");			break;	// shadow

	case 0x90: __dz80_op_plane(cpu, "SUB A,B");			break;	// shadow
	case 0x91: __dz80_op_plane(cpu, "SUB A,C");			break;	// shadow
	case 0x92: __dz80_op_plane(cpu, "SUB A,D");			break;	// shadow
	case 0x93: __dz80_op_plane(cpu, "SUB A,E");			break;	// shadow
	case 0x94: __dz80_op_plane(cpu, "SUB A,IYH");		break;
	case 0x95: __dz80_op_plane(cpu, "SUB A,IYL");		break;
	case 0x96: __dz80_op_ixd(cpu, "SUB A,(IY+d)");		break;
	case 0x97: __dz80_op_plane(cpu, "SUB A,A");			break;	// shadow

	case 0x98: __dz80_op_plane(cpu, "SBC A,B");			break;	// shadow
	case 0x99: __dz80_op_plane(cpu, "SBC A,C");			break;	// shadow
	case 0x9A: __dz80_op_plane(cpu, "SBC A,D");			break;	// shadow
	case 0x9B: __dz80_op_plane(cpu, "SBC A,E");			break;	// shadow
	case 0x9C: __dz80_op_plane(cpu, "SBC A,IYH");		break;
	case 0x9D: __dz80_op_plane(cpu, "SBC A,IYL");		break;
	case 0x9E: __dz80_op_ixd(cpu, "SBC A,(IY+d)");		break;
	case 0x9F: __dz80_op_plane(cpu, "SBC A,A");			break;	// shadow

	case 0xA0: __dz80_op_plane(cpu, "AND A,B");			break;	// shadow
	case 0xA1: __dz80_op_plane(cpu, "AND A,C");			break;	// shadow
	case 0xA2: __dz80_op_plane(cpu, "AND A,D");			break;	// shadow
	case 0xA3: __dz80_op_plane(cpu, "AND A,E");			break;	// shadow
	case 0xA4: __dz80_op_plane(cpu, "AND A,IYH");		break;
	case 0xA5: __dz80_op_plane(cpu, "AND A,IYL");		break;
	case 0xA6: __dz80_op_ixd(cpu, "AND A,(IY+d)");		break;
	case 0xA7: __dz80_op_plane(cpu, "AND A,A");			break;	// shadow

	case 0xA8: __dz80_op_plane(cpu, "XOR A,B");			break;	// shadow
	case 0xA9: __dz80_op_plane(cpu, "XOR A,C");			break;	// shadow
	case 0xAA: __dz80_op_plane(cpu, "XOR A,D");			break;	// shadow
	case 0xAB: __dz80_op_plane(cpu, "XOR A,E");			break;	// shadow
	case 0xAC: __dz80_op_plane(cpu, "XOR A,IYH");		break;
	case 0xAD: __dz80_op_plane(cpu, "XOR A,IYL");		break;
	case 0xAE: __dz80_op_ixd(cpu, "XOR A,(IY+d)");		break;
	case 0xAF: __dz80_op_plane(cpu, "XOR A,A");			break;	// shadow

	case 0xB0: __dz80_op_plane(cpu, "OR A,B");			break;	// shadow
	case 0xB1: __dz80_op_plane(cpu, "OR A,C");			break;	// shadow
	case 0xB2: __dz80_op_plane(cpu, "OR A,D");			break;	// shadow
	case 0xB3: __dz80_op_plane(cpu, "OR A,E");			break;	// shadow
	case 0xB4: __dz80_op_plane(cpu, "OR A,IYH");		break;
	case 0xB5: __dz80_op_plane(cpu, "OR A,IYL");		break;
	case 0xB6: __dz80_op_ixd(cpu, "OR A,(IY+d)");		break;
	case 0xB7: __dz80_op_plane(cpu, "OR A,A");			break;	// shadow

	case 0xB8: __dz80_op_plane(cpu, "CP A,B");			break;	// shadow
	case 0xB9: __dz80_op_plane(cpu, "CP A,C");			break;	// shadow
	case 0xBA: __dz80_op_plane(cpu, "CP A,D");			break;	// shadow
	case 0xBB: __dz80_op_plane(cpu, "CP A,E");			break;	// shadow
	case 0xBC: __dz80_op_plane(cpu, "CP IYH");			break;
	case 0xBD: __dz80_op_plane(cpu, "CP IYL");			break;
	case 0xBE: __dz80_op_ixd(cpu, "CP (IY+d)");			break;
	case 0xBF: __dz80_op_plane(cpu, "CP A,A");			break;	// shadow

	case 0xCB:	_dz80_decoder_fd_cb(cpu); 				break;

	case 0xE1: __dz80_op_plane(cpu, "POP IY");			break;
	case 0xE3: __dz80_op_plane(cpu, "EX (SP),IY");		break;
	case 0xE5: __dz80_op_plane(cpu, "PUSH IY");			break;
	case 0xE9: __dz80_op_plane(cpu, "JP (IY)");			break;

	case 0xF9: __dz80_op_plane(cpu, "LD SP,IY");		break;
	default: break;
	}
}

#ifdef __cplusplus
}	// extern "C"
#endif
